"""
This module provides Python wrappers for the GPU implementation in Blender.
Some higher level functions can be found in the gpu_extras module.

gpu.types.rst
gpu.matrix.rst
gpu.select.rst
gpu.shader.rst
gpu.state.rst
gpu.texture.rst
gpu.platform.rst
gpu.capabilities.rst

:maxdepth: 1
:caption: Submodules

"""

import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
from . import capabilities as capabilities
from . import matrix as matrix
from . import platform as platform
from . import select as select
from . import shader as shader
from . import state as state
from . import texture as texture
from . import types as types



"""
This module provides access to math operations.

[NOTE]
Classes, methods and attributes that accept vectors also accept other numeric sequences,
such as tuples, lists.

The mathutils module provides the following classes:

* Color,
* Euler,
* Matrix,
* Quaternion,
* Vector,

mathutils.geometry.rst
mathutils.bvhtree.rst
mathutils.kdtree.rst
mathutils.interpolate.rst
mathutils.noise.rst

:maxdepth: 1
:caption: Submodules

```../examples/mathutils.0.py```

"""

import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
from . import bvhtree as bvhtree
from . import geometry as geometry
from . import interpolate as interpolate
from . import kdtree as kdtree
from . import noise as noise

class Color:
    """This object gives access to Colors in Blender.Most colors returned by Blender APIs are in scene linear color space, as defined by the OpenColorIO configuration. The notable exception is user interface theming colors, which are in sRGB color space."""

    b: float
    """ Blue color channel."""

    g: float
    """ Green color channel."""

    h: float
    """ HSV Hue component in [0, 1]."""

    hsv: tuple[float, float, float]
    """ HSV Values in [0, 1]."""

    is_frozen: bool
    """ True when this object has been frozen (read-only)."""

    is_valid: bool
    """ True when the owner of this data is valid."""

    is_wrapped: bool
    """ True when this object wraps external data (read-only)."""

    owner: typing.Any
    """ The item this is wrapping or None (read-only)."""

    r: float
    """ Red color channel."""

    s: float
    """ HSV Saturation component in [0, 1]."""

    v: float
    """ HSV Value component in [0, 1]."""

    def copy(self) -> typing_extensions.Self:
        """Returns a copy of this color.

        :return: A copy of the color.
        """

    def freeze(self) -> typing_extensions.Self:
        """Make this object immutable.After this the object can be hashed, used in dictionaries & sets.

        :return: An instance of this object.
        """

    def from_aces_to_scene_linear(self) -> typing_extensions.Self:
        """Convert from ACES2065-1 linear to scene linear color space.

        :return: A color in scene linear color space.
        """

    def from_acescg_to_scene_linear(self) -> typing_extensions.Self:
        """Convert from ACEScg linear to scene linear color space.

        :return: A color in scene linear color space.
        """

    def from_rec2020_linear_to_scene_linear(self) -> typing_extensions.Self:
        """Convert from Rec.2020 linear color space to scene linear color space.

        :return: A color in scene linear color space.
        """

    def from_rec709_linear_to_scene_linear(self) -> typing_extensions.Self:
        """Convert from Rec.709 linear color space to scene linear color space.

        :return: A color in scene linear color space.
        """

    def from_scene_linear_to_aces(self) -> typing_extensions.Self:
        """Convert from scene linear to ACES2065-1 linear color space.

        :return: A color in ACES2065-1 linear color space.
        """

    def from_scene_linear_to_acescg(self) -> typing_extensions.Self:
        """Convert from scene linear to ACEScg linear color space.

        :return: A color in ACEScg linear color space.
        """

    def from_scene_linear_to_rec2020_linear(self) -> typing_extensions.Self:
        """Convert from scene linear to Rec.2020 linear color space.

        :return: A color in Rec.2020 linear color space.
        """

    def from_scene_linear_to_rec709_linear(self) -> typing_extensions.Self:
        """Convert from scene linear to Rec.709 linear color space.

        :return: A color in Rec.709 linear color space.
        """

    def from_scene_linear_to_srgb(self) -> typing_extensions.Self:
        """Convert from scene linear to sRGB color space.

        :return: A color in sRGB color space.
        """

    def from_scene_linear_to_xyz_d65(self) -> typing_extensions.Self:
        """Convert from scene linear to CIE XYZ (Illuminant D65) color space.

        :return: A color in XYZ color space.
        """

    def from_srgb_to_scene_linear(self) -> typing_extensions.Self:
        """Convert from sRGB to scene linear color space.

        :return: A color in scene linear color space.
        """

    def from_xyz_d65_to_scene_linear(self) -> typing_extensions.Self:
        """Convert from CIE XYZ (Illuminant D65) to scene linear color space.

        :return: A color in scene linear color space.
        """

    def __init__(self, rgb: collections.abc.Sequence[float] = (0.0, 0.0, 0.0)) -> None:
        """

        :param rgb:
        """

    def __get__(self, instance, owner) -> typing_extensions.Self:
        """

        :param instance:
        :param owner:
        :return:
        """

    def __set__(
        self, instance, value: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> None:
        """

        :param instance:
        :param value:
        """

    def __add__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __sub__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __mul__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __truediv__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __radd__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __rsub__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __rmul__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __rtruediv__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __iadd__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __isub__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __imul__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __itruediv__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    @typing.overload
    def __getitem__(self, key: int) -> float:
        """

        :param key:
        :return:
        """

    @typing.overload
    def __getitem__(self, key: slice) -> tuple[float, ...]:
        """

        :param key:
        :return:
        """

    @typing.overload
    def __setitem__(self, key: int, value: float) -> None:
        """

        :param key:
        :param value:
        """

    @typing.overload
    def __setitem__(
        self, key: slice, value: collections.abc.Iterable[float] | Color
    ) -> None:
        """

        :param key:
        :param value:
        """

class Euler:
    """This object gives access to Eulers in Blender.`Euler angles <https://en.wikipedia.org/wiki/Euler_angles>`__ on Wikipedia."""

    is_frozen: bool
    """ True when this object has been frozen (read-only)."""

    is_valid: bool
    """ True when the owner of this data is valid."""

    is_wrapped: bool
    """ True when this object wraps external data (read-only)."""

    order: typing.Literal["XYZ", "XZY", "YXZ", "YZX", "ZXY", "ZYX"]
    """ Euler rotation order."""

    owner: typing.Any
    """ The item this is wrapping or None (read-only)."""

    x: float
    """ Euler axis angle in radians."""

    y: float
    """ Euler axis angle in radians."""

    z: float
    """ Euler axis angle in radians."""

    def copy(self) -> typing_extensions.Self:
        """Returns a copy of this euler.

        :return: A copy of the euler.
        """

    def freeze(self) -> typing_extensions.Self:
        """Make this object immutable.After this the object can be hashed, used in dictionaries & sets.

        :return: An instance of this object.
        """

    def make_compatible(
        self,
        other: collections.abc.Sequence[float] | typing_extensions.Self,
        /,
    ) -> None:
        """Make this euler compatible with another,
        so interpolating between them works as intended.

                :param other: Other euler rotation.
        """

    def rotate(
        self,
        other: Matrix
        | Quaternion
        | collections.abc.Sequence[collections.abc.Sequence[float]]
        | collections.abc.Sequence[float]
        | typing_extensions.Self,
        /,
    ) -> None:
        """Rotates the euler by another mathutils value.

        :param other: rotation component of mathutils value
        """

    def rotate_axis(
        self,
        axis: typing.Literal["X", "Y", "Z"],
        angle: float,
        /,
    ) -> None:
        """Rotates the euler a certain amount and returning a unique euler rotation
        (no 720 degree pitches).

                :param axis: An axis string.
                :param angle: angle in radians.
        """

    def to_matrix(self) -> Matrix:
        """Return a matrix representation of the euler.

        :return: A 3x3 rotation matrix representation of the euler.
        """

    def to_quaternion(self) -> Quaternion:
        """Return a quaternion representation of the euler.

        :return: Quaternion representation of the euler.
        """

    def zero(self) -> None:
        """Set all values to zero."""

    def __init__(
        self,
        angles: collections.abc.Sequence[float] = (0.0, 0.0, 0.0),
        order: str = "XYZ",
    ) -> None:
        """

        :param angles:
        :param order:
        """

    def __get__(self, instance, owner) -> typing_extensions.Self:
        """

        :param instance:
        :param owner:
        :return:
        """

    def __set__(
        self, instance, value: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> None:
        """

        :param instance:
        :param value:
        """

    @typing.overload
    def __getitem__(self, key: int) -> float:
        """

        :param key:
        :return:
        """

    @typing.overload
    def __getitem__(self, key: slice) -> tuple[float, ...]:
        """

        :param key:
        :return:
        """

    @typing.overload
    def __setitem__(self, key: int, value: float) -> None:
        """

        :param key:
        :param value:
        """

    @typing.overload
    def __setitem__(
        self, key: slice, value: collections.abc.Iterable[float] | Euler
    ) -> None:
        """

        :param key:
        :param value:
        """

class Matrix:
    """This object gives access to Matrices in Blender, supporting square and rectangular
    matrices from 2x2 up to 4x4.
    """

    col: typing.Any
    """ Access the matrix by columns, 3x3 and 4x4 only, (read-only)."""

    is_frozen: bool
    """ True when this object has been frozen (read-only)."""

    is_identity: bool
    """ True if this is an identity matrix (read-only)."""

    is_negative: bool
    """ True if this matrix results in a negative scale, 3x3 and 4x4 only, (read-only)."""

    is_orthogonal: bool
    """ True if this matrix is orthogonal, 3x3 and 4x4 only, (read-only)."""

    is_orthogonal_axis_vectors: bool
    """ True if this matrix has orthogonal axis vectors, 3x3 and 4x4 only, (read-only)."""

    is_valid: bool
    """ True when the owner of this data is valid."""

    is_wrapped: bool
    """ True when this object wraps external data (read-only)."""

    median_scale: float
    """ The average scale applied to each axis (read-only)."""

    owner: typing.Any
    """ The item this is wrapping or None (read-only)."""

    row: typing.Any
    """ Access the matrix by rows (default), (read-only)."""

    translation: Vector
    """ The translation component of the matrix."""

    @classmethod
    def Diagonal(
        cls,
        vector: Vector | collections.abc.Sequence[float],
        /,
    ) -> typing_extensions.Self:
        """Create a diagonal (scaling) matrix using the values from the vector.

        :param vector: The vector of values for the diagonal.
        :return: A diagonal matrix.
        """

    @classmethod
    def Identity(
        cls,
        size: int,
        /,
    ) -> typing_extensions.Self:
        """Create an identity matrix.

        :param size: The size of the identity matrix to construct [2, 4].
        :return: A new identity matrix.
        """

    @classmethod
    def LocRotScale(
        cls,
        location: None | Vector | collections.abc.Sequence[float],
        rotation: Euler
        | None
        | Quaternion
        | collections.abc.Sequence[collections.abc.Sequence[float]]
        | collections.abc.Sequence[float]
        | typing_extensions.Self,
        scale: None | Vector | collections.abc.Sequence[float],
        /,
    ) -> typing_extensions.Self:
        """Create a matrix combining translation, rotation and scale,
        acting as the inverse of the decompose() method.Any of the inputs may be replaced with None if not needed.

                :param location: The translation component.
                :param rotation: The rotation component as a 3x3 matrix, quaternion, euler or None for no rotation.
                :param scale: The scale component.
                :return: Combined transformation as a 4x4 matrix.
        """

    @classmethod
    def OrthoProjection(
        cls,
        axis: Vector
        | collections.abc.Sequence[float]
        | typing.Literal["X", "Y", "XY", "XZ", "YZ"],
        size: int,
        /,
    ) -> typing_extensions.Self:
        """Create a matrix to represent an orthographic projection.

                :param axis: An axis string,
        where a single axis is for a 2D matrix.
        Or a vector for an arbitrary axis
                :param size: The size of the projection matrix to construct [2, 4].
                :return: A new projection matrix.
        """

    @classmethod
    def Rotation(
        cls,
        angle: float,
        size: int,
        axis: Vector
        | collections.abc.Sequence[float]
        | typing.Literal["X", "Y", "Z"]
        | None = [],
        /,
    ) -> typing_extensions.Self:
        """Create a matrix representing a rotation.

                :param angle: The angle of rotation desired, in radians.
                :param size: The size of the rotation matrix to construct [2, 4].
                :param axis: an axis string or a 3D Vector Object
        (optional when size is 2).
                :return: A new rotation matrix.
        """

    @classmethod
    def Scale(
        cls,
        factor: float,
        size: int,
        axis: Vector | collections.abc.Sequence[float] | None = [],
        /,
    ) -> typing_extensions.Self:
        """Create a matrix representing a scaling.

        :param factor: The factor of scaling to apply.
        :param size: The size of the scale matrix to construct [2, 4].
        :param axis: Direction to influence scale. (optional).
        :return: A new scale matrix.
        """

    @classmethod
    def Shear(
        cls,
        plane: typing.Literal["X", "Y", "XY", "XZ", "YZ"],
        size: int,
        factor: collections.abc.Sequence[float] | float,
        /,
    ) -> typing_extensions.Self:
        """Create a matrix to represent a shear transformation.

                :param plane: An axis string,
        where a single axis is for a 2D matrix only.
                :param size: The size of the shear matrix to construct [2, 4].
                :param factor: The factor of shear to apply. For a 2 size matrix use a single float. For a 3 or 4 size matrix pass a pair of floats corresponding with the plane axis.
                :return: A new shear matrix.
        """

    @classmethod
    def Translation(
        cls,
        vector: Vector | collections.abc.Sequence[float],
        /,
    ) -> typing_extensions.Self:
        """Create a matrix representing a translation.

        :param vector: The translation vector.
        :return: An identity matrix with a translation.
        """

    def adjugate(self) -> None:
        """Set the matrix to its adjugate.`Adjugate matrix <https://en.wikipedia.org/wiki/Adjugate_matrix>`__ on Wikipedia."""

    def adjugated(self) -> typing_extensions.Self:
        """Return an adjugated copy of the matrix.

        :return: the adjugated matrix.
        """

    def copy(self) -> typing_extensions.Self:
        """Returns a copy of this matrix.

        :return: an instance of itself
        """

    def decompose(self) -> tuple[Vector, Quaternion, Vector]:
        """Return the translation, rotation, and scale components of this matrix.

        :return: Tuple of translation, rotation, and scale.
        """

    def determinant(self) -> float:
        """Return the determinant of a matrix.`Determinant <https://en.wikipedia.org/wiki/Determinant>`__ on Wikipedia.

        :return: Return the determinant of a matrix.
        """

    def freeze(self) -> typing_extensions.Self:
        """Make this object immutable.After this the object can be hashed, used in dictionaries & sets.

        :return: An instance of this object.
        """

    def identity(self) -> None:
        """Set the matrix to the identity matrix.`Identity matrix <https://en.wikipedia.org/wiki/Identity_matrix>`__ on Wikipedia."""

    def invert(
        self,
        fallback: None
        | collections.abc.Sequence[collections.abc.Sequence[float]]
        | typing_extensions.Self
        | None = None,
        /,
    ) -> None:
        """Set the matrix to its inverse.`Inverse matrix <https://en.wikipedia.org/wiki/Inverse_matrix>`__ on Wikipedia.

                :param fallback: Set the matrix to this value when the inverse cannot be calculated
        (instead of raising a `ValueError` exception).
        """

    def invert_safe(self) -> None:
        """Set the matrix to its inverse, will never error.
        If degenerated (e.g. zero scale on an axis), add some epsilon to its diagonal, to get an invertible one.
        If tweaked matrix is still degenerated, set to the identity matrix instead.`Inverse Matrix <https://en.wikipedia.org/wiki/Inverse_matrix>`__ on Wikipedia.

        """

    def inverted(
        self,
        fallback: typing.Any | None = None,
        /,
    ) -> typing.Any | typing_extensions.Self:
        """Return an inverted copy of the matrix.

                :param fallback: return this when the inverse cant be calculated
        (instead of raising a `ValueError`).
                :return: The inverted matrix or fallback when given.
        """

    def inverted_safe(self) -> typing_extensions.Self:
        """Return an inverted copy of the matrix, will never error.
        If degenerated (e.g. zero scale on an axis), add some epsilon to its diagonal, to get an invertible one.
        If tweaked matrix is still degenerated, return the identity matrix instead.

                :return: the inverted matrix.
        """

    def lerp(
        self,
        other: collections.abc.Sequence[collections.abc.Sequence[float]]
        | typing_extensions.Self,
        factor: float,
        /,
    ) -> typing_extensions.Self:
        """Returns the interpolation of two matrices. Uses polar decomposition, see "Matrix Animation and Polar Decomposition", Shoemake and Duff, 1992.

        :param other: value to interpolate with.
        :param factor: The interpolation value in [0.0, 1.0].
        :return: The interpolated matrix.
        """

    def normalize(self) -> None:
        """Normalize each of the matrix columns."""

    def normalized(self) -> typing_extensions.Self:
        """Return a column normalized matrix

        :return: a column normalized matrix
        """

    def resize_4x4(self) -> None:
        """Resize the matrix to 4x4."""

    def rotate(
        self,
        other: Euler
        | Quaternion
        | collections.abc.Sequence[collections.abc.Sequence[float]]
        | collections.abc.Sequence[float]
        | typing_extensions.Self,
        /,
    ) -> None:
        """Rotates the matrix by another mathutils value.

        :param other: rotation component of mathutils value
        """

    def to_2x2(self) -> typing_extensions.Self:
        """Return a 2x2 copy of this matrix.

        :return: a new matrix.
        """

    def to_3x3(self) -> typing_extensions.Self:
        """Return a 3x3 copy of this matrix.

        :return: a new matrix.
        """

    def to_4x4(self) -> typing_extensions.Self:
        """Return a 4x4 copy of this matrix.

        :return: a new matrix.
        """

    def to_euler(
        self,
        order: typing.Literal["XYZ", "XZY", "YXZ", "YZX", "ZXY", "ZYX"] = "XYZ",
        euler_compat: Euler | None | collections.abc.Sequence[float] | None = None,
        /,
    ) -> Euler:
        """Return an Euler representation of the rotation matrix
        (3x3 or 4x4 matrix only).

                :param order: A rotation order string.
                :param euler_compat: Optional euler argument the new euler will be made
        compatible with (no axis flipping between them).
        Useful for converting a series of matrices to animation curves.
                :return: Euler representation of the matrix.
        """

    def to_quaternion(self) -> Quaternion:
        """Return a quaternion representation of the rotation matrix.

        :return: Quaternion representation of the rotation matrix.
        """

    def to_scale(self) -> Vector:
        """Return the scale part of a 3x3 or 4x4 matrix.

        :return: Return the scale of a matrix.
        """

    def to_translation(self) -> Vector:
        """Return the translation part of a 4 row matrix.

        :return: Return the translation of a matrix.
        """

    def transpose(self) -> None:
        """Set the matrix to its transpose.`Transpose <https://en.wikipedia.org/wiki/Transpose>`__ on Wikipedia."""

    def transposed(self) -> typing_extensions.Self:
        """Return a new, transposed matrix.

        :return: a transposed matrix
        """

    def zero(self) -> None:
        """Set all the matrix values to zero."""

    def __init__(
        self,
        rows: collections.abc.Sequence[collections.abc.Sequence[float]] = (
            (1.0, 0.0, 0.0, 0.0),
            (0.0, 1.0, 0.0, 0.0),
            (0.0, 0.0, 1.0, 0.0),
            (0.0, 0.0, 0.0, 1.0),
        ),
    ) -> None:
        """

        :param rows:
        """

    def __get__(self, instance, owner) -> typing_extensions.Self:
        """

        :param instance:
        :param owner:
        :return:
        """

    def __set__(
        self,
        instance,
        value: collections.abc.Sequence[collections.abc.Sequence[float]]
        | typing_extensions.Self,
    ) -> None:
        """

        :param instance:
        :param value:
        """

    @typing.overload
    def __getitem__(self, key: int) -> Vector:
        """

        :param key:
        :return:
        """

    @typing.overload
    def __getitem__(self, key: slice) -> tuple[Vector, ...]:
        """

        :param key:
        :return:
        """

    @typing.overload
    def __setitem__(
        self, key: int, value: Vector | collections.abc.Iterable[float]
    ) -> None:
        """

        :param key:
        :param value:
        """

    @typing.overload
    def __setitem__(
        self,
        key: slice,
        value: collections.abc.Iterable[Vector | collections.abc.Iterable[float]]
        | Matrix,
    ) -> None:
        """

        :param key:
        :param value:
        """

    def __len__(self) -> int:
        """

        :return:
        """

    def __add__(
        self,
        other: collections.abc.Sequence[collections.abc.Sequence[float]]
        | typing_extensions.Self,
    ) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __sub__(
        self,
        other: collections.abc.Sequence[collections.abc.Sequence[float]]
        | typing_extensions.Self,
    ) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __mul__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    @typing.overload
    def __matmul__(
        self,
        other: collections.abc.Sequence[collections.abc.Sequence[float]]
        | typing_extensions.Self,
    ) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    @typing.overload
    def __matmul__(self, other: Vector | collections.abc.Sequence[float]) -> Vector:
        """

        :param other:
        :return:
        """

    def __radd__(
        self,
        other: collections.abc.Sequence[collections.abc.Sequence[float]]
        | typing_extensions.Self,
    ) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __rsub__(
        self,
        other: collections.abc.Sequence[collections.abc.Sequence[float]]
        | typing_extensions.Self,
    ) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __rmul__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __imul__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

class Quaternion:
    """This object gives access to Quaternions in Blender.The constructor takes arguments in various forms:"""

    angle: float
    """ Angle of the quaternion."""

    axis: Vector
    """ Quaternion axis as a vector."""

    is_frozen: bool
    """ True when this object has been frozen (read-only)."""

    is_valid: bool
    """ True when the owner of this data is valid."""

    is_wrapped: bool
    """ True when this object wraps external data (read-only)."""

    magnitude: float
    """ Size of the quaternion (read-only)."""

    owner: typing.Any
    """ The item this is wrapping or None (read-only)."""

    w: float
    """ Quaternion component value."""

    x: float
    """ Quaternion component value."""

    y: float
    """ Quaternion component value."""

    z: float
    """ Quaternion component value."""

    def conjugate(self) -> None:
        """Set the quaternion to its conjugate (negate x, y, z)."""

    def conjugated(self) -> typing_extensions.Self:
        """Return a new conjugated quaternion.

        :return: a new quaternion.
        """

    def copy(self) -> typing_extensions.Self:
        """Returns a copy of this quaternion.

        :return: A copy of the quaternion.
        """

    def cross(
        self,
        other: collections.abc.Sequence[float] | typing_extensions.Self,
        /,
    ) -> typing_extensions.Self:
        """Return the cross product of this quaternion and another.

        :param other: The other quaternion to perform the cross product with.
        :return: The cross product.
        """

    def dot(
        self,
        other: collections.abc.Sequence[float] | typing_extensions.Self,
        /,
    ) -> float:
        """Return the dot product of this quaternion and another.

        :param other: The other quaternion to perform the dot product with.
        :return: The dot product.
        """

    def freeze(self) -> typing_extensions.Self:
        """Make this object immutable.After this the object can be hashed, used in dictionaries & sets.

        :return: An instance of this object.
        """

    def identity(self) -> None:
        """Set the quaternion to an identity quaternion."""

    def invert(self) -> None:
        """Set the quaternion to its inverse."""

    def inverted(self) -> typing_extensions.Self:
        """Return a new, inverted quaternion.

        :return: the inverted value.
        """

    def make_compatible(
        self,
        other: collections.abc.Sequence[float] | typing_extensions.Self,
        /,
    ) -> None:
        """Make this quaternion compatible with another,
        so interpolating between them works as intended.

                :param other: The other quaternion to make compatible with.
        """

    def negate(self) -> None:
        """Set the quaternion to its negative."""

    def normalize(self) -> None:
        """Normalize the quaternion."""

    def normalized(self) -> typing_extensions.Self:
        """Return a new normalized quaternion.

        :return: a normalized copy.
        """

    def rotate(
        self,
        other: Euler
        | Matrix
        | collections.abc.Sequence[collections.abc.Sequence[float]]
        | collections.abc.Sequence[float]
        | typing_extensions.Self,
        /,
    ) -> None:
        """Rotates the quaternion by another mathutils value.

        :param other: rotation component of mathutils value
        """

    def rotation_difference(
        self,
        other: collections.abc.Sequence[float] | typing_extensions.Self,
        /,
    ) -> typing_extensions.Self:
        """Returns a quaternion representing the rotational difference.

        :param other: second quaternion.
        :return: the rotational difference between the two quat rotations.
        """

    def slerp(
        self,
        other: collections.abc.Sequence[float] | typing_extensions.Self,
        factor: float,
        /,
    ) -> typing_extensions.Self:
        """Returns the interpolation of two quaternions.

        :param other: value to interpolate with.
        :param factor: The interpolation value in [0.0, 1.0].
        :return: The interpolated rotation.
        """

    def to_axis_angle(self) -> tuple[Vector, float]:
        """Return the axis, angle representation of the quaternion.

        :return: Axis, angle.
        """

    def to_euler(
        self,
        order: typing.Literal["XYZ", "XZY", "YXZ", "YZX", "ZXY", "ZYX"] = "XYZ",
        euler_compat: Euler | None | collections.abc.Sequence[float] | None = None,
        /,
    ) -> Euler:
        """Return Euler representation of the quaternion.

                :param order: Rotation order.
                :param euler_compat: Optional euler argument the new euler will be made
        compatible with (no axis flipping between them).
        Useful for converting a series of quaternions to animation curves.
                :return: Euler representation of the quaternion.
        """

    def to_exponential_map(self) -> Vector:
        """Return the exponential map representation of the quaternion.This representation consists of the rotation axis multiplied by the rotation angle.
        Such a representation is useful for interpolation between multiple orientations.To convert back to a quaternion, pass it to the `Quaternion` constructor.

                :return: 3D exponential map.
        """

    def to_matrix(self) -> Matrix:
        """Return a matrix representation of the quaternion.

        :return: A 3x3 rotation matrix representation of the quaternion.
        """

    def to_swing_twist(
        self,
        axis: typing.Literal["X", "Y", "Z"],
        /,
    ) -> tuple[Quaternion, float]:
        """Split the rotation into a swing quaternion with the specified
        axis fixed at zero, and the remaining twist rotation angle.

                :param axis: Twist axis as a string.
                :return: Swing, twist angle.
        """

    def __init__(
        self,
        seq: Vector | collections.abc.Sequence[float] = (1.0, 0.0, 0.0, 0.0),
        angle: float = 0.0,
    ) -> None:
        """

        :param seq:
        :param angle:
        """

    def __get__(self, instance, owner) -> typing_extensions.Self:
        """

        :param instance:
        :param owner:
        :return:
        """

    def __set__(
        self, instance, value: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> None:
        """

        :param instance:
        :param value:
        """

    def __len__(self) -> int:
        """

        :return:
        """

    @typing.overload
    def __getitem__(self, key: int) -> float:
        """

        :param key:
        :return:
        """

    @typing.overload
    def __getitem__(self, key: slice) -> tuple[float, ...]:
        """

        :param key:
        :return:
        """

    @typing.overload
    def __setitem__(self, key: int, value: float) -> None:
        """

        :param key:
        :param value:
        """

    @typing.overload
    def __setitem__(
        self, key: slice, value: collections.abc.Iterable[float] | Quaternion
    ) -> None:
        """

        :param key:
        :param value:
        """

    def __add__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __sub__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __mul__(
        self, other: collections.abc.Sequence[float] | float | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    @typing.overload
    def __matmul__(self, other: typing_extensions.Self) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    @typing.overload
    def __matmul__(self, other: Vector) -> Vector:
        """

        :param other:
        :return:
        """

    def __radd__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __rsub__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __rmul__(
        self, other: collections.abc.Sequence[float] | float | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __imul__(
        self, other: collections.abc.Sequence[float] | float | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

class Vector:
    """This object gives access to Vectors in Blender."""

    is_frozen: bool
    """ True when this object has been frozen (read-only)."""

    is_valid: bool
    """ True when the owner of this data is valid."""

    is_wrapped: bool
    """ True when this object wraps external data (read-only)."""

    length: float
    """ Vector Length."""

    length_squared: float
    """ Vector length squared (v.dot(v))."""

    magnitude: float
    """ Vector Length."""

    owner: typing.Any
    """ The item this is wrapping or None (read-only)."""

    w: float
    """ Vector W axis (4D Vectors only)."""

    ww: typing_extensions.Self
    www: typing_extensions.Self
    wwww: typing_extensions.Self
    wwwx: typing_extensions.Self
    wwwy: typing_extensions.Self
    wwwz: typing_extensions.Self
    wwx: typing_extensions.Self
    wwxw: typing_extensions.Self
    wwxx: typing_extensions.Self
    wwxy: typing_extensions.Self
    wwxz: typing_extensions.Self
    wwy: typing_extensions.Self
    wwyw: typing_extensions.Self
    wwyx: typing_extensions.Self
    wwyy: typing_extensions.Self
    wwyz: typing_extensions.Self
    wwz: typing_extensions.Self
    wwzw: typing_extensions.Self
    wwzx: typing_extensions.Self
    wwzy: typing_extensions.Self
    wwzz: typing_extensions.Self
    wx: typing_extensions.Self
    wxw: typing_extensions.Self
    wxww: typing_extensions.Self
    wxwx: typing_extensions.Self
    wxwy: typing_extensions.Self
    wxwz: typing_extensions.Self
    wxx: typing_extensions.Self
    wxxw: typing_extensions.Self
    wxxx: typing_extensions.Self
    wxxy: typing_extensions.Self
    wxxz: typing_extensions.Self
    wxy: typing_extensions.Self
    wxyw: typing_extensions.Self
    wxyx: typing_extensions.Self
    wxyy: typing_extensions.Self
    wxyz: typing_extensions.Self
    wxz: typing_extensions.Self
    wxzw: typing_extensions.Self
    wxzx: typing_extensions.Self
    wxzy: typing_extensions.Self
    wxzz: typing_extensions.Self
    wy: typing_extensions.Self
    wyw: typing_extensions.Self
    wyww: typing_extensions.Self
    wywx: typing_extensions.Self
    wywy: typing_extensions.Self
    wywz: typing_extensions.Self
    wyx: typing_extensions.Self
    wyxw: typing_extensions.Self
    wyxx: typing_extensions.Self
    wyxy: typing_extensions.Self
    wyxz: typing_extensions.Self
    wyy: typing_extensions.Self
    wyyw: typing_extensions.Self
    wyyx: typing_extensions.Self
    wyyy: typing_extensions.Self
    wyyz: typing_extensions.Self
    wyz: typing_extensions.Self
    wyzw: typing_extensions.Self
    wyzx: typing_extensions.Self
    wyzy: typing_extensions.Self
    wyzz: typing_extensions.Self
    wz: typing_extensions.Self
    wzw: typing_extensions.Self
    wzww: typing_extensions.Self
    wzwx: typing_extensions.Self
    wzwy: typing_extensions.Self
    wzwz: typing_extensions.Self
    wzx: typing_extensions.Self
    wzxw: typing_extensions.Self
    wzxx: typing_extensions.Self
    wzxy: typing_extensions.Self
    wzxz: typing_extensions.Self
    wzy: typing_extensions.Self
    wzyw: typing_extensions.Self
    wzyx: typing_extensions.Self
    wzyy: typing_extensions.Self
    wzyz: typing_extensions.Self
    wzz: typing_extensions.Self
    wzzw: typing_extensions.Self
    wzzx: typing_extensions.Self
    wzzy: typing_extensions.Self
    wzzz: typing_extensions.Self
    x: float
    """ Vector X axis."""

    xw: typing_extensions.Self
    xww: typing_extensions.Self
    xwww: typing_extensions.Self
    xwwx: typing_extensions.Self
    xwwy: typing_extensions.Self
    xwwz: typing_extensions.Self
    xwx: typing_extensions.Self
    xwxw: typing_extensions.Self
    xwxx: typing_extensions.Self
    xwxy: typing_extensions.Self
    xwxz: typing_extensions.Self
    xwy: typing_extensions.Self
    xwyw: typing_extensions.Self
    xwyx: typing_extensions.Self
    xwyy: typing_extensions.Self
    xwyz: typing_extensions.Self
    xwz: typing_extensions.Self
    xwzw: typing_extensions.Self
    xwzx: typing_extensions.Self
    xwzy: typing_extensions.Self
    xwzz: typing_extensions.Self
    xx: typing_extensions.Self
    xxw: typing_extensions.Self
    xxww: typing_extensions.Self
    xxwx: typing_extensions.Self
    xxwy: typing_extensions.Self
    xxwz: typing_extensions.Self
    xxx: typing_extensions.Self
    xxxw: typing_extensions.Self
    xxxx: typing_extensions.Self
    xxxy: typing_extensions.Self
    xxxz: typing_extensions.Self
    xxy: typing_extensions.Self
    xxyw: typing_extensions.Self
    xxyx: typing_extensions.Self
    xxyy: typing_extensions.Self
    xxyz: typing_extensions.Self
    xxz: typing_extensions.Self
    xxzw: typing_extensions.Self
    xxzx: typing_extensions.Self
    xxzy: typing_extensions.Self
    xxzz: typing_extensions.Self
    xy: typing_extensions.Self
    xyw: typing_extensions.Self
    xyww: typing_extensions.Self
    xywx: typing_extensions.Self
    xywy: typing_extensions.Self
    xywz: typing_extensions.Self
    xyx: typing_extensions.Self
    xyxw: typing_extensions.Self
    xyxx: typing_extensions.Self
    xyxy: typing_extensions.Self
    xyxz: typing_extensions.Self
    xyy: typing_extensions.Self
    xyyw: typing_extensions.Self
    xyyx: typing_extensions.Self
    xyyy: typing_extensions.Self
    xyyz: typing_extensions.Self
    xyz: typing_extensions.Self
    xyzw: typing_extensions.Self
    xyzx: typing_extensions.Self
    xyzy: typing_extensions.Self
    xyzz: typing_extensions.Self
    xz: typing_extensions.Self
    xzw: typing_extensions.Self
    xzww: typing_extensions.Self
    xzwx: typing_extensions.Self
    xzwy: typing_extensions.Self
    xzwz: typing_extensions.Self
    xzx: typing_extensions.Self
    xzxw: typing_extensions.Self
    xzxx: typing_extensions.Self
    xzxy: typing_extensions.Self
    xzxz: typing_extensions.Self
    xzy: typing_extensions.Self
    xzyw: typing_extensions.Self
    xzyx: typing_extensions.Self
    xzyy: typing_extensions.Self
    xzyz: typing_extensions.Self
    xzz: typing_extensions.Self
    xzzw: typing_extensions.Self
    xzzx: typing_extensions.Self
    xzzy: typing_extensions.Self
    xzzz: typing_extensions.Self
    y: float
    """ Vector Y axis."""

    yw: typing_extensions.Self
    yww: typing_extensions.Self
    ywww: typing_extensions.Self
    ywwx: typing_extensions.Self
    ywwy: typing_extensions.Self
    ywwz: typing_extensions.Self
    ywx: typing_extensions.Self
    ywxw: typing_extensions.Self
    ywxx: typing_extensions.Self
    ywxy: typing_extensions.Self
    ywxz: typing_extensions.Self
    ywy: typing_extensions.Self
    ywyw: typing_extensions.Self
    ywyx: typing_extensions.Self
    ywyy: typing_extensions.Self
    ywyz: typing_extensions.Self
    ywz: typing_extensions.Self
    ywzw: typing_extensions.Self
    ywzx: typing_extensions.Self
    ywzy: typing_extensions.Self
    ywzz: typing_extensions.Self
    yx: typing_extensions.Self
    yxw: typing_extensions.Self
    yxww: typing_extensions.Self
    yxwx: typing_extensions.Self
    yxwy: typing_extensions.Self
    yxwz: typing_extensions.Self
    yxx: typing_extensions.Self
    yxxw: typing_extensions.Self
    yxxx: typing_extensions.Self
    yxxy: typing_extensions.Self
    yxxz: typing_extensions.Self
    yxy: typing_extensions.Self
    yxyw: typing_extensions.Self
    yxyx: typing_extensions.Self
    yxyy: typing_extensions.Self
    yxyz: typing_extensions.Self
    yxz: typing_extensions.Self
    yxzw: typing_extensions.Self
    yxzx: typing_extensions.Self
    yxzy: typing_extensions.Self
    yxzz: typing_extensions.Self
    yy: typing_extensions.Self
    yyw: typing_extensions.Self
    yyww: typing_extensions.Self
    yywx: typing_extensions.Self
    yywy: typing_extensions.Self
    yywz: typing_extensions.Self
    yyx: typing_extensions.Self
    yyxw: typing_extensions.Self
    yyxx: typing_extensions.Self
    yyxy: typing_extensions.Self
    yyxz: typing_extensions.Self
    yyy: typing_extensions.Self
    yyyw: typing_extensions.Self
    yyyx: typing_extensions.Self
    yyyy: typing_extensions.Self
    yyyz: typing_extensions.Self
    yyz: typing_extensions.Self
    yyzw: typing_extensions.Self
    yyzx: typing_extensions.Self
    yyzy: typing_extensions.Self
    yyzz: typing_extensions.Self
    yz: typing_extensions.Self
    yzw: typing_extensions.Self
    yzww: typing_extensions.Self
    yzwx: typing_extensions.Self
    yzwy: typing_extensions.Self
    yzwz: typing_extensions.Self
    yzx: typing_extensions.Self
    yzxw: typing_extensions.Self
    yzxx: typing_extensions.Self
    yzxy: typing_extensions.Self
    yzxz: typing_extensions.Self
    yzy: typing_extensions.Self
    yzyw: typing_extensions.Self
    yzyx: typing_extensions.Self
    yzyy: typing_extensions.Self
    yzyz: typing_extensions.Self
    yzz: typing_extensions.Self
    yzzw: typing_extensions.Self
    yzzx: typing_extensions.Self
    yzzy: typing_extensions.Self
    yzzz: typing_extensions.Self
    z: float
    """ Vector Z axis (3D Vectors only)."""

    zw: typing_extensions.Self
    zww: typing_extensions.Self
    zwww: typing_extensions.Self
    zwwx: typing_extensions.Self
    zwwy: typing_extensions.Self
    zwwz: typing_extensions.Self
    zwx: typing_extensions.Self
    zwxw: typing_extensions.Self
    zwxx: typing_extensions.Self
    zwxy: typing_extensions.Self
    zwxz: typing_extensions.Self
    zwy: typing_extensions.Self
    zwyw: typing_extensions.Self
    zwyx: typing_extensions.Self
    zwyy: typing_extensions.Self
    zwyz: typing_extensions.Self
    zwz: typing_extensions.Self
    zwzw: typing_extensions.Self
    zwzx: typing_extensions.Self
    zwzy: typing_extensions.Self
    zwzz: typing_extensions.Self
    zx: typing_extensions.Self
    zxw: typing_extensions.Self
    zxww: typing_extensions.Self
    zxwx: typing_extensions.Self
    zxwy: typing_extensions.Self
    zxwz: typing_extensions.Self
    zxx: typing_extensions.Self
    zxxw: typing_extensions.Self
    zxxx: typing_extensions.Self
    zxxy: typing_extensions.Self
    zxxz: typing_extensions.Self
    zxy: typing_extensions.Self
    zxyw: typing_extensions.Self
    zxyx: typing_extensions.Self
    zxyy: typing_extensions.Self
    zxyz: typing_extensions.Self
    zxz: typing_extensions.Self
    zxzw: typing_extensions.Self
    zxzx: typing_extensions.Self
    zxzy: typing_extensions.Self
    zxzz: typing_extensions.Self
    zy: typing_extensions.Self
    zyw: typing_extensions.Self
    zyww: typing_extensions.Self
    zywx: typing_extensions.Self
    zywy: typing_extensions.Self
    zywz: typing_extensions.Self
    zyx: typing_extensions.Self
    zyxw: typing_extensions.Self
    zyxx: typing_extensions.Self
    zyxy: typing_extensions.Self
    zyxz: typing_extensions.Self
    zyy: typing_extensions.Self
    zyyw: typing_extensions.Self
    zyyx: typing_extensions.Self
    zyyy: typing_extensions.Self
    zyyz: typing_extensions.Self
    zyz: typing_extensions.Self
    zyzw: typing_extensions.Self
    zyzx: typing_extensions.Self
    zyzy: typing_extensions.Self
    zyzz: typing_extensions.Self
    zz: typing_extensions.Self
    zzw: typing_extensions.Self
    zzww: typing_extensions.Self
    zzwx: typing_extensions.Self
    zzwy: typing_extensions.Self
    zzwz: typing_extensions.Self
    zzx: typing_extensions.Self
    zzxw: typing_extensions.Self
    zzxx: typing_extensions.Self
    zzxy: typing_extensions.Self
    zzxz: typing_extensions.Self
    zzy: typing_extensions.Self
    zzyw: typing_extensions.Self
    zzyx: typing_extensions.Self
    zzyy: typing_extensions.Self
    zzyz: typing_extensions.Self
    zzz: typing_extensions.Self
    zzzw: typing_extensions.Self
    zzzx: typing_extensions.Self
    zzzy: typing_extensions.Self
    zzzz: typing_extensions.Self

    @classmethod
    def Fill(
        cls,
        size: int,
        fill: float = 0.0,
        /,
    ) -> typing_extensions.Self:
        """Create a vector of length size with all values set to fill.

        :param size: The length of the vector to be created.
        :param fill: The value used to fill the vector.
        :return: A new vector.
        """

    @classmethod
    def Linspace(
        cls,
        start: float,
        stop: float,
        size: int,
        /,
    ) -> typing_extensions.Self:
        """Create a vector of the specified size which is filled with linearly spaced values between start and stop values.

        :param start: The start of the range used to fill the vector.
        :param stop: The end of the range used to fill the vector.
        :param size: The size of the vector to be created.
        :return: A new vector.
        """

    @classmethod
    def Range(
        cls,
        start: int,
        stop: int,
        step: int = 1,
        /,
    ) -> typing_extensions.Self:
        """Create a vector filled with a range of values.This method can also be called with a single argument, in which case the argument is interpreted as stop and start defaults to 0.

        :param start: The start of the range used to fill the vector.
        :param stop: The end of the range used to fill the vector.
        :param step: The step between successive values in the vector.
        :return: A new vector.
        """

    @classmethod
    def Repeat(
        cls,
        vector: collections.abc.Sequence[float] | typing_extensions.Self,
        size: int,
        /,
    ) -> typing_extensions.Self:
        """Create a vector by repeating the values in vector until the required size is reached.

        :param vector: The vector to draw values from.
        :param size: The size of the vector to be created.
        :return: A new vector.
        """

    def angle(
        self,
        other: collections.abc.Sequence[float] | typing_extensions.Self,
        fallback: typing.Any | None = None,
        /,
    ) -> float | typing.Any:
        """Return the angle between two vectors.

                :param other: another vector to compare the angle with
                :param fallback: return this when the angle cant be calculated (zero length vector),
        (instead of raising a `ValueError`).
                :return: angle in radians or fallback when given
        """

    def angle_signed(
        self,
        other: collections.abc.Sequence[float] | typing_extensions.Self,
        fallback: typing.Any | None = None,
        /,
    ) -> float | typing.Any:
        """Return the signed angle between two 2D vectors (clockwise is positive).

                :param other: another vector to compare the angle with
                :param fallback: return this when the angle cant be calculated (zero length vector),
        (instead of raising a `ValueError`).
                :return: angle in radians or fallback when given
        """

    def copy(self) -> typing_extensions.Self:
        """Returns a copy of this vector.

        :return: A copy of the vector.
        """

    def cross(
        self,
        other: collections.abc.Sequence[float] | typing_extensions.Self,
        /,
    ) -> float | typing_extensions.Self:
        """Return the cross product of this vector and another.

        :param other: The other vector to perform the cross product with.
        :return: The cross product as a vector or a float when 2D vectors are used.
        """

    def dot(
        self,
        other: collections.abc.Sequence[float] | typing_extensions.Self,
        /,
    ) -> float:
        """Return the dot product of this vector and another.

        :param other: The other vector to perform the dot product with.
        :return: The dot product.
        """

    def freeze(self) -> typing_extensions.Self:
        """Make this object immutable.After this the object can be hashed, used in dictionaries & sets.

        :return: An instance of this object.
        """

    def lerp(
        self,
        other: collections.abc.Sequence[float] | typing_extensions.Self,
        factor: float,
        /,
    ) -> typing_extensions.Self:
        """Returns the interpolation of two vectors.

        :param other: value to interpolate with.
        :param factor: The interpolation value in [0.0, 1.0].
        :return: The interpolated vector.
        """

    def negate(self) -> None:
        """Set all values to their negative."""

    def normalize(self) -> None:
        """Normalize the vector, making the length of the vector always 1.0."""

    def normalized(self) -> typing_extensions.Self:
        """Return a new, normalized vector.

        :return: a normalized copy of the vector
        """

    def orthogonal(self) -> typing_extensions.Self:
        """Return a perpendicular vector.

        :return: a new vector 90 degrees from this vector.
        """

    def project(
        self,
        other: collections.abc.Sequence[float] | typing_extensions.Self,
        /,
    ) -> typing_extensions.Self:
        """Return the projection of this vector onto the other.

        :param other: second vector.
        :return: the parallel projection vector
        """

    def reflect(
        self,
        mirror: collections.abc.Sequence[float] | typing_extensions.Self,
        /,
    ) -> typing_extensions.Self:
        """Return the reflection vector from the mirror argument.

        :param mirror: This vector could be a normal from the reflecting surface.
        :return: The reflected vector matching the size of this vector.
        """

    def resize(
        self,
        size: int,
        /,
    ) -> None:
        """Resize the vector to have size number of elements.

        :param size: The new size of the vector.
        """

    def resize_2d(self) -> None:
        """Resize the vector to 2D (x, y)."""

    def resize_3d(self) -> None:
        """Resize the vector to 3D (x, y, z)."""

    def resize_4d(self) -> None:
        """Resize the vector to 4D (x, y, z, w)."""

    def resized(
        self,
        size: int,
        /,
    ) -> typing_extensions.Self:
        """Return a resized copy of the vector with size number of elements.

        :param size: The new size of the vector.
        :return: A new vector.
        """

    def rotate(
        self,
        other: Euler
        | Matrix
        | Quaternion
        | collections.abc.Sequence[collections.abc.Sequence[float]]
        | collections.abc.Sequence[float],
        /,
    ) -> None:
        """Rotate the vector by a rotation value.

        :param other: rotation component of mathutils value
        """

    def rotation_difference(
        self,
        other: collections.abc.Sequence[float] | typing_extensions.Self,
        /,
    ) -> Quaternion:
        """Returns a quaternion representing the rotational difference between this
        vector and another.

                :param other: second vector.
                :return: the rotational difference between the two vectors.
        """

    def slerp(
        self,
        other: collections.abc.Sequence[float] | typing_extensions.Self,
        factor: float,
        fallback: typing.Any | None = None,
        /,
    ) -> typing_extensions.Self:
        """Returns the interpolation of two non-zero vectors (spherical coordinates).

                :param other: value to interpolate with.
                :param factor: The interpolation value typically in [0.0, 1.0].
                :param fallback: return this when the vector cant be calculated (zero length vector or direct opposites),
        (instead of raising a `ValueError`).
                :return: The interpolated vector.
        """

    def to_2d(self) -> typing_extensions.Self:
        """Return a 2d copy of the vector.

        :return: a new vector
        """

    def to_3d(self) -> typing_extensions.Self:
        """Return a 3d copy of the vector.

        :return: a new vector
        """

    def to_4d(self) -> typing_extensions.Self:
        """Return a 4d copy of the vector.

        :return: a new vector
        """

    def to_track_quat(
        self,
        track: typing.Literal["X", "Y", "Z", "-X", "-Y", "-Z"] = "Z",
        up: typing.Literal["X", "Y", "Z"] = "Y",
        /,
    ) -> Quaternion:
        """Return a quaternion rotation from the vector and the track and up axis.

        :param track: Track axis string.
        :param up: Up axis string.
        :return: rotation from the vector and the track and up axis.
        """

    def to_tuple(
        self,
        precision: int = -1,
        /,
    ) -> tuple[float, ...]:
        """Return this vector as a tuple with a given precision.

        :param precision: The number to round the value to in [-1, 21].
        :return: the values of the vector rounded by precision
        """

    def zero(self) -> None:
        """Set all values to zero."""

    def __init__(self, seq: collections.abc.Sequence[float] = (0.0, 0.0, 0.0)) -> None:
        """

        :param seq:
        """

    def __get__(self, instance, owner) -> typing_extensions.Self:
        """

        :param instance:
        :param owner:
        :return:
        """

    def __set__(
        self, instance, value: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> None:
        """

        :param instance:
        :param value:
        """

    def __len__(self) -> int:
        """

        :return:
        """

    @typing.overload
    def __getitem__(self, key: int) -> float:
        """

        :param key:
        :return:
        """

    @typing.overload
    def __getitem__(self, key: slice) -> tuple[float, ...]:
        """

        :param key:
        :return:
        """

    @typing.overload
    def __setitem__(self, key: int, value: float) -> None:
        """

        :param key:
        :param value:
        """

    @typing.overload
    def __setitem__(
        self, key: slice, value: collections.abc.Iterable[float] | Vector
    ) -> None:
        """

        :param key:
        :param value:
        """

    def __neg__(self) -> typing_extensions.Self:
        """

        :return:
        """

    def __add__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __sub__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    @typing.overload
    def __mul__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    @typing.overload
    def __mul__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __truediv__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    @typing.overload
    def __matmul__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> float:
        """

        :param other:
        :return:
        """

    @typing.overload
    def __matmul__(
        self, other: Matrix | collections.abc.Sequence[collections.abc.Sequence[float]]
    ) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __radd__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __rsub__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __rmul__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __rtruediv__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __iadd__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __isub__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __imul__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """

    def __itruediv__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :return:
        """



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt

class AssetLibraryMeta:
    """AssetLibraryMeta(api_versions: dict[str, str], name: str, contact: Contact)"""

class Contact:
    """Contact(name: str, url: str | None = None, email: str | None = None)"""

    email: typing.Any
    url: typing.Any



"""
[NOTE]
All subscribers will be cleared on file-load. Subscribers can be re-registered on load,
see bpy.app.handlers.load_post.

"""

import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import bpy.types

def clear_by_owner(owner: typing.Any | None) -> None:
    """Clear all subscribers using this owner.

    :param owner: The owner handle passed to `subscribe_rna`.
    """

def publish_rna(
    key: bpy.types.Property | bpy.types.Struct | tuple[bpy.types.Struct, str] | None,
) -> None:
    """Notify subscribers of changes to this property
    (this typically doesnt need to be called explicitly since changes will automatically publish updates).
    In some cases it may be useful to publish changes explicitly using more general keys.

        :param key: Represents the type of data being subscribed to

    Arguments include
    - A property instance.
    - A struct type.
    - A tuple representing a (struct, property name) pair.
    """

def subscribe_rna(
    key: bpy.types.Property | bpy.types.Struct | tuple[bpy.types.Struct, str] | None,
    owner: typing.Any | None,
    args: tuple | None,
    notify: collections.abc.Callable[..., None] | None,
    *,
    options: set[typing.Literal["PERSISTENT"]] | None = set(),
) -> None:
    """Register a message bus subscription. It will be cleared when another blend file is
    loaded, or can be cleared explicitly via `bpy.msgbus.clear_by_owner`.

        :param key: Represents the type of data being subscribed to

    Arguments include
    - A property instance.
    - A struct type.
    - A tuple representing a (struct, property name) pair.
        :param owner: Handle for this subscription (compared by identity).
        :param args: Arguments passed to the callback.
        :param notify: The callback function.
        :param options: Change the behavior of the subscriber.

    PERSISTENT when set, the subscriber will be kept when remapping ID data.
    """



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import bpy.types
import mathutils

class AddObjectHelper:
    def align_update_callback(self, _context) -> None:
        """Update callback for the align property, resets rotation for world alignment.

        :param _context:
        """

def add_object_align_init(
    context: bpy.types.Context, operator: None | bpy.types.Operator
) -> mathutils.Matrix:
    """Return a matrix using the operator settings and view context.

    :param context: The context to use.
    :param operator: The operator, checked for location and rotation properties.
    :return: the matrix from the context and settings.
    """

def object_add_grid_scale(context: bpy.types.Context) -> float:
    """Return scale which should be applied on object
    data to align it to grid scale.

        :param context: The context.
        :return: The grid scale.
    """

def object_add_grid_scale_apply_operator(
    operator: bpy.types.Operator, context: bpy.types.Context
) -> None:
    """Scale an operators distance values by the grid size.

    :param operator: The operator to scale.
    :param context: The context.
    """

def object_data_add(
    context: bpy.types.Context,
    obdata: None | bpy.types.ID,
    operator: None | bpy.types.Operator | None = None,
    name: None | str | None = None,
) -> bpy.types.Object:
    """Add an object using the view context and preference to initialize the
    location, rotation and layer.

        :param context: The context to use.
        :param obdata: Valid object data to be used for the new object or None.
        :param operator: The operator, checked for location and rotation properties.
        :param name: Optional name
        :return: the newly created object in the scene.
    """

def object_report_if_active_shape_key_is_locked(
    obj: bpy.types.Object, operator: None | bpy.types.Operator
) -> bool:
    """Checks if the active shape key of the specified object is locked, and reports an error if so.If the object has no shape keys, there is nothing to lock, and the function returns False.

    :param obj: Object to check.
    :param operator: Currently running operator to report the error through. Use None to suppress emitting the message.
    :return: True if the shape key was locked.
    """

def world_to_camera_view(
    scene: bpy.types.Scene,
    obj: bpy.types.Object,
    coord: collections.abc.Sequence[float] | mathutils.Vector,
) -> mathutils.Vector:
    """Returns the camera space coords for a 3d point.
    (also known as: normalized device coordinates - NDC).Where (0, 0) is the bottom left and (1, 1)
    is the top right of the camera frame.
    values outside 0-1 are also supported.
    A negative z value means the point is behind the camera.Takes shift-x/y, lens angle and sensor size into account
    as well as perspective/ortho projections.

        :param scene: Scene to use for frame size.
        :param obj: Camera object.
        :param coord: World space location.
        :return: a vector where X and Y map to the view plane and
    Z is the depth on the view axis.
    """



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import bpy.types

def connect_sockets(input: bpy.types.NodeSocket, output: bpy.types.NodeSocket) -> None:
    """Connect sockets in a node tree.This is useful because the links created through the normal Python API are
    invalid when one of the sockets is a virtual socket (grayed out sockets in
    Group Input and Group Output nodes).It replaces node_tree.links.new(input, output)

        :param input: The input socket.
        :param output: The output socket.
    """

def find_base_socket_type(socket: bpy.types.NodeSocket) -> str:
    """Find the base class of the socket.Sockets can have a subtype such as NodeSocketFloatFactor,
    but only the base type is allowed, e.g. NodeSocketFloat

        :param socket: The socket to find the base type for.
        :return: The base socket type identifier.
    """

def find_node_input(node: bpy.types.Node, name: str) -> None | bpy.types.NodeSocket:
    """Find a node input socket by name.Note that names are not unique, returns the first match.

    :param node: The node to search.
    :param name: The name of the input socket.
    :return: The input socket or None if not found.
    """



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt

def new(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> None:
    """Add a new texture"""

def slot_copy(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> None:
    """Copy the material texture settings and nodes"""

def slot_move(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: typing.Literal["UP", "DOWN"] | None = "UP",
) -> None:
    """Move texture slots up and down

    :param type: Type
    """

def slot_paste(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> None:
    """Copy the texture settings and nodes"""



"""
Helpers for asset management tasks.

"""

import typing
import collections.abc
import typing_extensions
import numpy.typing as npt

class AssetBrowserPanel:
    """Mixin class for panels that should only show in the asset browser."""

class AssetMetaDataPanel:
    """Mixin class for panels that display asset metadata in the asset browser."""

class SpaceAssetInfo:
    """Utility class for checking if a space is an asset browser."""



"""
This module provides access to GPUShader internal functions.

Built-in shaders

All built-in shaders have the mat4 ModelViewProjectionMatrix

 uniform.

Its value must be modified using the gpu.matrix module.

[IMPORTANT]
Shader uniforms must be explicitly initialized to avoid retaining values from previous executions.

FLAT_COLOR


    Attributes: vec3 pos, vec4 color
    Uniforms: none



IMAGE


    Attributes: vec3 pos, vec2 texCoord
    Uniforms: sampler2D image



IMAGE_SCENE_LINEAR_TO_REC709_SRGB


    Attributes: vec3 pos, vec2 texCoord
    Uniforms: sampler2D image
    Note: Expect texture to be in scene linear color space



IMAGE_COLOR


    Attributes: vec3 pos, vec2 texCoord
    Uniforms: sampler2D image, vec4 color



IMAGE_COLOR_SCENE_LINEAR_TO_REC709_SRGB


    Attributes: vec3 pos, vec2 texCoord
    Uniforms: sampler2D image, vec4 color
    Note: Expect texture to be in scene linear color space



SMOOTH_COLOR


    Attributes: vec3 pos, vec4 color
    Uniforms: none



UNIFORM_COLOR


    Attributes: vec3 pos
    Uniforms: vec4 color



POLYLINE_FLAT_COLOR


    Attributes: vec3 pos, vec4 color
    Uniforms: vec2 viewportSize, float lineWidth



POLYLINE_SMOOTH_COLOR


    Attributes: vec3 pos, vec4 color
    Uniforms: vec2 viewportSize, float lineWidth



POLYLINE_UNIFORM_COLOR


    Attributes: vec3 pos
    Uniforms: vec2 viewportSize, float lineWidth, vec4 color



POINT_FLAT_COLOR


    Attributes: vec3 pos, vec4 color
    Uniforms: float size



POINT_UNIFORM_COLOR


    Attributes: vec3 pos
    Uniforms: vec4 color, float size



"""

import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import gpu.types

def create_from_info(shader_info: gpu.types.GPUShaderCreateInfo) -> gpu.types.GPUShader:
    """Create shader from a GPUShaderCreateInfo.

    :param shader_info: GPUShaderCreateInfo
    :return: Shader object corresponding to the given shader info.
    """

def from_builtin(shader_name: str, *, config: str = "DEFAULT") -> gpu.types.GPUShader:
    """Shaders that are embedded in the Blender internal code (see `built-in-shaders`).
    They all read the uniform mat4 ModelViewProjectionMatrix,
    which can be edited by the `gpu.matrix` module.You can also choose a shader configuration that uses clip_planes by setting the CLIPPED value to the config parameter. Note that in this case you also need to manually set the value of mat4 ModelMatrix.

        :param shader_name: One of the builtin shader names.
        :param config: One of these types of shader configuration:

    DEFAULT

    CLIPPED
        :return: Shader object corresponding to the given name.
    """

def unbind() -> None:
    """Unbind the bound shader object."""



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt

class DiskFileHashBackend:
    """Base class for protocol classes.Protocol classes are defined as:Such classes are primarily used with static type checkers that recognize
    structural subtyping (static duck-typing).For example:See PEP 544 for details. Protocol classes decorated with
    @typing.runtime_checkable act as simple-minded runtime protocols that check
    only the presence of given attributes, ignoring their type signatures.
    Protocol classes can be generic, they are defined as:
    """

    def close(self) -> None:
        """Close the back-end.After calling this, the back-end is not expected to work any more."""

    def fetch_hash(self, filepath, hash_algorithm) -> None:
        """Return the cached hash info of a given file.If no info is cached for this path/algorithm combo, returns None.

        :param filepath:
        :param hash_algorithm:
        """

    def mark_hash_as_fresh(self, filepath, hash_algorithm) -> None:
        """Store that the hash is still considered fresh.See remove_older_than().

        :param filepath:
        :param hash_algorithm:
        """

    def open(self) -> None:
        """Prepare the back-end for use."""

    def remove_older_than(self, *, days) -> None:
        """Remove all hash entries that are older than this many days.When this removes all known hashes for a file, the file entry itself is
        also removed.

                :param days:
        """

    def store_hash(
        self, filepath, hash_algorithm, hash_info, pre_write_callback=None
    ) -> None:
        """Store a pre-computed hash for the given file path.See DiskFileHashService.store_hash() for an explanation of the parameters.

        :param filepath:
        :param hash_algorithm:
        :param hash_info:
        :param pre_write_callback:
        """

class FileHashInfo:
    """FileHashInfo(hexhash: str, file_size_bytes: int, file_stat_mtime: float)"""



"""
Generic 3-dimensional kd-tree to perform spatial searches.

```../examples/mathutils.kdtree.0.py```

"""

import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import mathutils

class KDTree:
    """KDTree(size) -> new kd-tree initialized to hold size items."""

    def balance(self) -> None:
        """Balance the tree."""

    def find(
        self,
        co: collections.abc.Sequence[float],
        *,
        filter: None | collections.abc.Callable[int, bool] | None = None,
    ) -> tuple[mathutils.Vector, int, float]:
        """Find nearest point to co.

        :param co: 3D coordinate.
        :param filter: function which takes an index and returns True for indices to include in the search.
        :return: Returns (position, index, distance).
        """

    def find_n(
        self, co: collections.abc.Sequence[float], n: int
    ) -> list[tuple[mathutils.Vector, int, float]]:
        """Find nearest n points to co.

        :param co: 3D coordinates.
        :param n: Number of points to find.
        :return: Returns a list of tuples (position, index, distance).
        """

    def find_range(
        self, co: collections.abc.Sequence[float], radius: float
    ) -> list[tuple[mathutils.Vector, int, float]]:
        """Find all points within radius of co.

        :param co: 3D coordinates.
        :param radius: Distance to search for points.
        :return: Returns a list of tuples (position, index, distance).
        """

    def insert(self, co: collections.abc.Sequence[float], index: int) -> None:
        """Insert a point into the KDTree.

        :param co: Point 3d position.
        :param index: The index of the point.
        """

    def __init__(self, size) -> None:
        """

        :param size:
        """



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import _bpy_types
import bpy.types

class TOPBAR_HT_upper_bar(_bpy_types.Header):
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

    def draw_left(self, context) -> None:
        """

        :param context:
        """

    def draw_right(self, context) -> None:
        """

        :param context:
        """

class TOPBAR_MT_blender(_bpy_types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, _context) -> None:
        """

        :param _context:
        """

class TOPBAR_MT_blender_system(_bpy_types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, _context) -> None:
        """

        :param _context:
        """

class TOPBAR_MT_edit(_bpy_types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

class TOPBAR_MT_editor_menus(_bpy_types.Menu):
    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

class TOPBAR_MT_file(_bpy_types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

class TOPBAR_MT_file_cleanup(_bpy_types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, _context) -> None:
        """

        :param _context:
        """

class TOPBAR_MT_file_context_menu(_bpy_types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, _context) -> None:
        """

        :param _context:
        """

class TOPBAR_MT_file_defaults(_bpy_types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

class TOPBAR_MT_file_export(_bpy_types.Menu):
    bl_idname: typing.Any
    bl_label: typing.Any
    bl_owner_use_filter: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

class TOPBAR_MT_file_external_data(_bpy_types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, _context) -> None:
        """

        :param _context:
        """

class TOPBAR_MT_file_import(_bpy_types.Menu):
    bl_idname: typing.Any
    bl_label: typing.Any
    bl_owner_use_filter: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

class TOPBAR_MT_file_new(_bpy_types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    @staticmethod
    def app_template_paths() -> None: ...
    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

    @staticmethod
    def draw_ex(layout, _context, *, use_splash=False, use_more=False) -> None:
        """

        :param layout:
        :param _context:
        :param use_splash:
        :param use_more:
        """

class TOPBAR_MT_file_previews(_bpy_types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, _context) -> None:
        """

        :param _context:
        """

class TOPBAR_MT_file_recover(_bpy_types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, _context) -> None:
        """

        :param _context:
        """

class TOPBAR_MT_help(_bpy_types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

class TOPBAR_MT_render(_bpy_types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

class TOPBAR_MT_templates_more(_bpy_types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

class TOPBAR_MT_window(_bpy_types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

class TOPBAR_MT_workspace_menu(_bpy_types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, _context) -> None:
        """

        :param _context:
        """

class TOPBAR_PT_gpencil_primitive(_bpy_types.Panel):
    bl_label: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

class TOPBAR_PT_grease_pencil_layers(_bpy_types.Panel):
    bl_label: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    bl_ui_units_x: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

    @classmethod
    def poll(cls, context) -> None:
        """

        :param context:
        """

class TOPBAR_PT_name(_bpy_types.Panel):
    bl_label: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    bl_ui_units_x: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

class TOPBAR_PT_name_marker(_bpy_types.Panel):
    bl_label: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    bl_ui_units_x: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

    @staticmethod
    def get_selected_marker(context) -> None:
        """

        :param context:
        """

    @staticmethod
    def is_using_pose_markers(context) -> None:
        """

        :param context:
        """

    @staticmethod
    def is_using_sequencer(context) -> None:
        """

        :param context:
        """

    @staticmethod
    def row_with_icon(layout, icon) -> None:
        """

        :param layout:
        :param icon:
        """

class TOPBAR_PT_tool_fallback(_bpy_types.Panel):
    bl_label: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    bl_ui_units_x: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

class TOPBAR_PT_tool_settings_extra(_bpy_types.Panel):
    """Popover panel for adding extra options that dont fit in the tool settings header"""

    bl_description: typing.Any
    bl_idname: typing.Any
    bl_label: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt

def add(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> None:
    """Add new cache"""

def bake(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    bake: bool | None = False,
) -> None:
    """Bake physics

    :param bake: Bake
    """

def bake_all(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    bake: bool | None = True,
) -> None:
    """Bake all physics

    :param bake: Bake
    """

def bake_from_cache(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> None:
    """Bake from cache"""

def free_bake(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> None:
    """Delete physics bake"""

def free_bake_all(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> None:
    """Delete all baked caches of all objects in the current scene"""

def remove(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> None:
    """Delete current cache"""



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import bpy.ops.transform

def attribute_set(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    value_float: float | None = 0.0,
    value_float_vector_2d: collections.abc.Iterable[float] | None = (0.0, 0.0),
    value_float_vector_3d: collections.abc.Iterable[float] | None = (0.0, 0.0, 0.0),
    value_int: int | None = 0,
    value_int_vector_2d: collections.abc.Iterable[int] | None = (0, 0),
    value_color: collections.abc.Iterable[float] | None = (1.0, 1.0, 1.0, 1.0),
    value_bool: bool | None = False,
) -> None:
    """Set values of the active attribute for selected elements

    :param value_float: Value
    :param value_float_vector_2d: Value
    :param value_float_vector_3d: Value
    :param value_int: Value
    :param value_int_vector_2d: Value
    :param value_color: Value
    :param value_bool: Value
    """

def delete(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> None:
    """Remove selected points"""

def duplicate(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> None:
    """Copy selected points"""

def duplicate_move(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    POINTCLOUD_OT_duplicate: duplicate | None = None,
    TRANSFORM_OT_translate: bpy.ops.transform.translate | None = None,
) -> None:
    """Make copies of selected elements and move them

    :param POINTCLOUD_OT_duplicate: Duplicate, Copy selected points
    :param TRANSFORM_OT_translate: Move, Move selected items
    """

def select_all(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    action: typing.Literal["TOGGLE", "SELECT", "DESELECT", "INVERT"] | None = "TOGGLE",
) -> None:
    """(De)select all point cloud

        :param action: Action, Selection action to execute

    TOGGLE
    Toggle -- Toggle selection for all elements.

    SELECT
    Select -- Select all elements.

    DESELECT
    Deselect -- Deselect all elements.

    INVERT
    Invert -- Invert selection of all elements.
    """

def select_random(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    seed: int | None = 0,
    probability: float | None = 0.5,
) -> None:
    """Randomizes existing selection or create new random selection

    :param seed: Seed, Source of randomness
    :param probability: Probability, Chance of every point being included in the selection
    """

def separate(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> None:
    """Separate selected geometry into a new point cloud"""



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt

class RawBlendFileReader:
    """Return a file handle to the raw blend file data (abstracting compressed formats)."""

def get_render_info_structure(endian_str, size) -> None: ...
def main() -> None: ...
def read_blend_rend_chunk(filepath) -> None: ...



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import bpy.stub_internal.rna_enums

def rule_add(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: bpy.stub_internal.rna_enums.BoidruleTypeItems | None = "GOAL",
) -> None:
    """Add a boid rule to the current boid state

    :param type: Type
    """

def rule_del(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> None:
    """Delete current boid rule"""

def rule_move_down(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> None:
    """Move boid rule down in the list"""

def rule_move_up(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> None:
    """Move boid rule up in the list"""

def state_add(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> None:
    """Add a boid state to the particle system"""

def state_del(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> None:
    """Delete current boid state"""

def state_move_down(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> None:
    """Move boid state down in the list"""

def state_move_up(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> None:
    """Move boid state up in the list"""



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import _bpy_types
import bpy.types

class EditExternally(_bpy_types.Operator):
    """Edit image in an external application"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def execute(self, context) -> None:
        """

        :param context:
        """

    def invoke(self, context, _event) -> None:
        """

        :param context:
        :param _event:
        """

class IMAGE_FH_drop_handler(_bpy_types.FileHandler):
    bl_file_extensions: typing.Any
    bl_idname: typing.Any
    bl_import_operator: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    @classmethod
    def poll_drop(cls, context) -> None:
        """

        :param context:
        """

class IMAGE_OT_open_images(_bpy_types.Operator):
    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def execute(self, context) -> None:
        """

        :param context:
        """

    @classmethod
    def poll(cls, context) -> None:
        """

        :param context:
        """

class ProjectApply(_bpy_types.Operator):
    """Project edited image back onto the object"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def execute(self, _context) -> None:
        """

        :param _context:
        """

class ProjectEdit(_bpy_types.Operator):
    """Edit a snapshot of the 3D Viewport in an external image editor"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def execute(self, context) -> None:
        """

        :param context:
        """



"""
This module provides access to Blender's text drawing functions.


--------------------

Example of using the blf module. For this module to work we
need to use the GPU module gpu as well.

```../examples/blf.0.py```

"""

import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import imbuf.types

def aspect(fontid: int, aspect: float) -> None:
    """Set the aspect for drawing text.

    :param fontid: The id of the typeface as returned by `blf.load`, for default font use 0.
    :param aspect: The aspect ratio for text drawing to use.
    """

def bind_imbuf(fontid: int, imbuf: imbuf.types.ImBuf) -> None:
    """Context manager to draw text into an image buffer instead of the GPUs context.

    :param fontid: The id of the typeface as returned by `blf.load`, for default font use 0.
    :param imbuf: The image to draw into.
    :return: The BLF ImBuf context manager.
    """

def clipping(fontid: int, xmin: float, ymin: float, xmax: float, ymax: float) -> None:
    """Set the clipping, enable/disable using CLIPPING.

    :param fontid: The id of the typeface as returned by `blf.load`, for default font use 0.
    :param xmin: Clip the drawing area by these bounds.
    :param ymin: Clip the drawing area by these bounds.
    :param xmax: Clip the drawing area by these bounds.
    :param ymax: Clip the drawing area by these bounds.
    """

def color(fontid: int, r: float, g: float, b: float, a: float) -> None:
    """Set the color for drawing text.

    :param fontid: The id of the typeface as returned by `blf.load`, for default font use 0.
    :param r: red channel 0.0 - 1.0.
    :param g: green channel 0.0 - 1.0.
    :param b: blue channel 0.0 - 1.0.
    :param a: alpha channel 0.0 - 1.0.
    """

def dimensions(fontid: int, text: str) -> tuple[float, float]:
    """Return the width and height of the text.

    :param fontid: The id of the typeface as returned by `blf.load`, for default font use 0.
    :param text: the text to draw.
    :return: the width and height of the text.
    """

def disable(fontid: int, option: int) -> None:
    """Disable option.

    :param fontid: The id of the typeface as returned by `blf.load`, for default font use 0.
    :param option: One of ROTATION, CLIPPING, SHADOW, MONOCHROME or WORD_WRAP.
    """

def draw(fontid: int, text: str) -> None:
    """Draw text in the current context.

    :param fontid: The id of the typeface as returned by `blf.load`, for default font use 0.
    :param text: the text to draw.
    """

def draw_buffer(fontid: int, text: str) -> None:
    """Draw text into the buffer bound to the fontid.

    :param fontid: The id of the typeface as returned by `blf.load`, for default font use 0.
    :param text: the text to draw.
    """

def enable(fontid: int, option: int) -> None:
    """Enable option.

    :param fontid: The id of the typeface as returned by `blf.load`, for default font use 0.
    :param option: One of ROTATION, CLIPPING, SHADOW, MONOCHROME or WORD_WRAP.
    """

def load(filepath: bytes | str) -> int:
    """Load a new font.

    :param filepath: the filepath of the font.
    :return: the new fonts fontid or -1 if there was an error.
    """

def position(fontid: int, x: float, y: float, z: float) -> None:
    """Set the position for drawing text.

    :param fontid: The id of the typeface as returned by `blf.load`, for default font use 0.
    :param x: X axis position to draw the text.
    :param y: Y axis position to draw the text.
    :param z: Z axis position to draw the text.
    """

def rotation(fontid: int, angle: float) -> None:
    """Set the text rotation angle, enable/disable using ROTATION.

    :param fontid: The id of the typeface as returned by `blf.load`, for default font use 0.
    :param angle: The angle for text drawing to use.
    """

def shadow(fontid: int, level: int, r: float, g: float, b: float, a: float) -> None:
    """Shadow options, enable/disable using SHADOW.

    :param fontid: The id of the typeface as returned by `blf.load`, for default font use 0.
    :param level: The blur level (0, 3, 5) or outline (6).
    :param r: Shadow color (red channel 0.0 - 1.0).
    :param g: Shadow color (green channel 0.0 - 1.0).
    :param b: Shadow color (blue channel 0.0 - 1.0).
    :param a: Shadow color (alpha channel 0.0 - 1.0).
    """

def shadow_offset(fontid: int, x: int, y: int) -> None:
    """Set the offset for shadow text.

    :param fontid: The id of the typeface as returned by `blf.load`, for default font use 0.
    :param x: Horizontal shadow offset value in pixels.
    :param y: Vertical shadow offset value in pixels.
    """

def size(fontid: int, size: float) -> None:
    """Set the size for drawing text.

    :param fontid: The id of the typeface as returned by `blf.load`, for default font use 0.
    :param size: Point size of the font.
    """

def unload(filepath: bytes | str) -> None:
    """Unload an existing font.

    :param filepath: the filepath of the font.
    """

def word_wrap(fontid: int, wrap_width: int) -> None:
    """Set the wrap width, enable/disable using WORD_WRAP.

    :param fontid: The id of the typeface as returned by `blf.load`, for default font use 0.
    :param wrap_width: The width (in pixels) to wrap words at.
    """

CLIPPING: typing.Any
""" Constant value 2
"""

MONOCHROME: typing.Any
""" Constant value 128
"""

ROTATION: typing.Any
""" Constant value 1
"""

SHADOW: typing.Any
""" Constant value 4
"""

WORD_WRAP: typing.Any
""" Constant value 64
"""



"""
This module defines properties to extend Blender's internal data. The result of these functions is used to assign properties to classes registered with Blender and can't be used directly.

[NOTE]
All parameters to these functions must be passed as keywords.


--------------------

Custom properties can be added to any subclass of an ID,
Bone and PoseBone.

These properties can be animated, accessed by the user interface and Python
like Blender's existing properties.

[WARNING]
Access to these properties might happen in threaded context, on a per-data-block level.
This has to be carefully considered when using accessors or update callbacks.
Typically, these callbacks should not affect any other data that the one owned by their data-block.
When accessing external non-Blender data, thread safety mechanisms should be considered.

```../examples/bpy.props.0.py```

[NOTE]
Pointer properties do not support storing references to embedded IDs (e.g. bpy.types.Scene.collection, bpy.types.Material.node_tree).
These should exclusively be referenced and accessed through their owner ID (e.g. the scene or material).

"""

import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import bpy.stub_internal.rna_enums
import bpy.types

def BoolProperty[_GenericType1: bpy.types.bpy_struct](
    *,
    name: str | None = "",
    description: str | None = "",
    translation_context: str | None = "*",
    default: bool | None = False,
    options: set[bpy.stub_internal.rna_enums.PropertyFlagItems] = {"ANIMATABLE"},
    override: set[bpy.stub_internal.rna_enums.PropertyOverrideFlagItems] = set(),
    tags: set[str] | None = set(),
    subtype: bpy.stub_internal.rna_enums.PropertySubtypeNumberItems = "NONE",
    update: collections.abc.Callable[[_GenericType1, bpy.types.Context], None]
    | None = None,
    get: collections.abc.Callable[[_GenericType1], bool] | None = None,
    set: collections.abc.Callable[[_GenericType1, bool], None] | None = None,
    get_transform: None
    | collections.abc.Callable[[bpy.types.bpy_struct, bool, bool], bool]
    | None = None,
    set_transform: None
    | collections.abc.Callable[[bpy.types.bpy_struct, bool, bool, bool], bool]
    | None = None,
) -> None:
    """Returns a new boolean property definition.

        :param name: Name used in the user interface.
        :param description: Text used for the tooltip and api documentation.
        :param translation_context: Text used as context to disambiguate translations.
        :param default: The default value for this property.
        :param options: Enumerator in `rna_enum_property_flag_items`.
        :param override: Enumerator in `rna_enum_property_override_flag_items`.
        :param tags: Enumerator of tags that are defined by parent class.
        :param subtype: Enumerator in `rna_enum_property_subtype_number_items`.
        :param update: Function to be called when this value is modified,
    This function must take 2 values (self, context) and return None.
    Warning there are no safety checks to avoid infinite recursion.
        :param get: Function to be called when this value is read, and the default,
    system-defined storage is not used for this property.
    This function must take 1 value (self) and return the value of the property.

    Defining this callback without a matching set one will make the property read-only (even if READ_ONLY option is not set).
        :param set: Function to be called when this value is written, and the default,
    system-defined storage is not used for this property.
    This function must take 2 values (self, value) and return None.

    Defining this callback without a matching get one is invalid.
        :param get_transform: Function to be called when this value is read,
    if some additional processing must be performed on the stored value.
    This function must take three arguments (self, the stored value,
    and a boolean indicating if the property is currently set),
    and return the final, transformed value of the property.

    The callback is responsible to ensure that value limits of the property (min/max, length...) are respected. Otherwise a ValueError exception is raised.
        :param set_transform: Function to be called when this value is written,
    if some additional processing must be performed on the given value before storing it.
    This function must take four arguments (self, the given value to store,
    the currently stored value (raw value, without any get_transform applied to it),
    and a boolean indicating if the property is currently set),
    and return the final, transformed value of the property.

    The callback is responsible to ensure that value limits (min/max, length...) are respected. Otherwise a ValueError exception is raised.
    """

def BoolVectorProperty[_GenericType1: bpy.types.bpy_struct](
    *,
    name: str | None = "",
    description: str | None = "",
    translation_context: str | None = "*",
    default: collections.abc.Sequence[bool] | None = (False, False, False),
    options: set[bpy.stub_internal.rna_enums.PropertyFlagItems] = {"ANIMATABLE"},
    override: set[bpy.stub_internal.rna_enums.PropertyOverrideFlagItems] = set(),
    tags: set[str] | None = set(),
    subtype: bpy.stub_internal.rna_enums.PropertySubtypeNumberArrayItems = "NONE",
    size: collections.abc.Sequence[int] | int | None = 3,
    update: collections.abc.Callable[[_GenericType1, bpy.types.Context], None]
    | None = None,
    get: collections.abc.Callable[[_GenericType1], collections.abc.Sequence[bool]]
    | None = None,
    set: collections.abc.Callable[[_GenericType1, collections.abc.Sequence[bool]], None]
    | None = None,
    get_transform: None
    | collections.abc.Callable[
        [bpy.types.bpy_struct, collections.abc.Sequence[bool], bool],
        collections.abc.Sequence[bool],
    ]
    | None = None,
    set_transform: None
    | collections.abc.Callable[
        [
            bpy.types.bpy_struct,
            collections.abc.Sequence[bool],
            collections.abc.Sequence[bool],
            bool,
        ],
        collections.abc.Sequence[bool],
    ]
    | None = None,
) -> None:
    """Returns a new vector boolean property definition.

        :param name: Name used in the user interface.
        :param description: Text used for the tooltip and api documentation.
        :param translation_context: Text used as context to disambiguate translations.
        :param default: sequence of booleans the length of size.
        :param options: Enumerator in `rna_enum_property_flag_items`.
        :param override: Enumerator in `rna_enum_property_override_flag_items`.
        :param tags: Enumerator of tags that are defined by parent class.
        :param subtype: Enumerator in `rna_enum_property_subtype_number_array_items`.
        :param size: Vector dimensions in [1, 32]. An int sequence can be used to define multi-dimension arrays.
        :param update: Function to be called when this value is modified,
    This function must take 2 values (self, context) and return None.
    Warning there are no safety checks to avoid infinite recursion.
        :param get: Function to be called when this value is read, and the default,
    system-defined storage is not used for this property.
    This function must take 1 value (self) and return the value of the property.

    Defining this callback without a matching set one will make the property read-only (even if READ_ONLY option is not set).
        :param set: Function to be called when this value is written, and the default,
    system-defined storage is not used for this property.
    This function must take 2 values (self, value) and return None.

    Defining this callback without a matching get one is invalid.
        :param get_transform: Function to be called when this value is read,
    if some additional processing must be performed on the stored value.
    This function must take three arguments (self, the stored value,
    and a boolean indicating if the property is currently set),
    and return the final, transformed value of the property.

    The callback is responsible to ensure that value limits of the property (min/max, length...) are respected. Otherwise a ValueError exception is raised.
        :param set_transform: Function to be called when this value is written,
    if some additional processing must be performed on the given value before storing it.
    This function must take four arguments (self, the given value to store,
    the currently stored value (raw value, without any get_transform applied to it),
    and a boolean indicating if the property is currently set),
    and return the final, transformed value of the property.

    The callback is responsible to ensure that value limits (min/max, length...) are respected. Otherwise a ValueError exception is raised.
    """

def CollectionProperty(
    *,
    type: type[bpy.types.PropertyGroup] | None,
    name: str | None = "",
    description: str | None = "",
    translation_context: str | None = "*",
    options: set[bpy.stub_internal.rna_enums.PropertyFlagItems] = {"ANIMATABLE"},
    override: set[
        bpy.stub_internal.rna_enums.PropertyOverrideFlagCollectionItems
    ] = set(),
    tags: set[str] | None = set(),
) -> None:
    """Returns a new collection property definition.

    :param type: A subclass of a property group.
    :param name: Name used in the user interface.
    :param description: Text used for the tooltip and api documentation.
    :param translation_context: Text used as context to disambiguate translations.
    :param options: Enumerator in `rna_enum_property_flag_items`.
    :param override: Enumerator in `rna_enum_property_override_flag_collection_items`.
    :param tags: Enumerator of tags that are defined by parent class.
    """

def EnumProperty[_GenericType1: bpy.types.bpy_struct](
    *,
    items: collections.abc.Iterable[
        tuple[str, str, str]
        | tuple[str, str, str, int]
        | tuple[str, str, str, str, int]
        | None
    ]
    | collections.abc.Callable[
        [_GenericType1, bpy.types.Context | None],
        collections.abc.Iterable[
            tuple[str, str, str]
            | tuple[str, str, str, int]
            | tuple[str, str, str, str, int]
            | None
        ],
    ],
    name: str | None = "",
    description: str | None = "",
    translation_context: str | None = "*",
    default: None | int | set[str] | str | None = None,
    options: set[bpy.stub_internal.rna_enums.PropertyFlagEnumItems] = {"ANIMATABLE"},
    override: set[bpy.stub_internal.rna_enums.PropertyOverrideFlagItems] = set(),
    tags: set[str] | None = set(),
    update: collections.abc.Callable[[_GenericType1, bpy.types.Context], None]
    | None = None,
    get: collections.abc.Callable[[_GenericType1], int] | None = None,
    set: collections.abc.Callable[[_GenericType1, int], None] | None = None,
    get_transform: None
    | collections.abc.Callable[[bpy.types.bpy_struct, int, bool], int]
    | None = None,
    set_transform: None
    | collections.abc.Callable[[bpy.types.bpy_struct, int, int, bool], int]
    | None = None,
) -> None:
    """Returns a new enumerator property definition.

        :param items: sequence of enum items formatted:
    [(identifier, name, description, icon, number), ...].

    The first three elements of the tuples are mandatory.

    identifier

    The identifier is used for Python access.
    An empty identifier means that the item is a separator

    name

    Name for the interface.

    description

    Used for documentation and tooltips.

    icon

    An icon string identifier or integer icon value
    (e.g. returned by `bpy.types.UILayout.icon`)

    number

    Unique value used as the identifier for this item (stored in file data).
    Use when the identifier may need to change. If the ENUM_FLAG option is used,
    the values are bit-masks and should be powers of two.

    When an item only contains 4 items they define (identifier, name, description, number).

    Separators may be added using either None (nameless separator),
    or a regular item tuple with an empty identifier string, in which case the name,
    if non-empty, will be displayed in the UI above the separator line.
    For dynamic values a callback can be passed which returns a list in
    the same format as the static list.
    This function must take 2 arguments (self, context), context may be None.

    There is a known bug with using a callback,
    Python must keep a reference to the strings returned by the callback or Blender
    will misbehave or even crash.
        :param name: Name used in the user interface.
        :param description: Text used for the tooltip and api documentation.
        :param translation_context: Text used as context to disambiguate translations.
        :param default: The default value for this enum, a string from the identifiers used in items, or integer matching an item number.
    If the ENUM_FLAG option is used this must be a set of such string identifiers instead.
    WARNING: Strings cannot be specified for dynamic enums
    (i.e. if a callback function is given as items parameter).
        :param options: Enumerator in `rna_enum_property_flag_enum_items`.
        :param override: Enumerator in `rna_enum_property_override_flag_items`.
        :param tags: Enumerator of tags that are defined by parent class.
        :param update: Function to be called when this value is modified,
    This function must take 2 values (self, context) and return None.
    Warning there are no safety checks to avoid infinite recursion.
        :param get: Function to be called when this value is read, and the default,
    system-defined storage is not used for this property.
    This function must take 1 value (self) and return the value of the property.

    Defining this callback without a matching set one will make the property read-only (even if READ_ONLY option is not set).
        :param set: Function to be called when this value is written, and the default,
    system-defined storage is not used for this property.
    This function must take 2 values (self, value) and return None.

    Defining this callback without a matching get one is invalid.
        :param get_transform: Function to be called when this value is read,
    if some additional processing must be performed on the stored value.
    This function must take three arguments (self, the stored value,
    and a boolean indicating if the property is currently set),
    and return the final, transformed value of the property.

    The callback is responsible to ensure that value limits of the property (min/max, length...) are respected. Otherwise a ValueError exception is raised.
        :param set_transform: Function to be called when this value is written,
    if some additional processing must be performed on the given value before storing it.
    This function must take four arguments (self, the given value to store,
    the currently stored value (raw value, without any get_transform applied to it),
    and a boolean indicating if the property is currently set),
    and return the final, transformed value of the property.

    The callback is responsible to ensure that value limits (min/max, length...) are respected. Otherwise a ValueError exception is raised.
    """

def FloatProperty[_GenericType1: bpy.types.bpy_struct](
    *,
    name: str | None = "",
    description: str | None = "",
    translation_context: str | None = "*",
    default: float | None = 0.0,
    min: float | None = -3.402823e38,
    max: float | None = 3.402823e38,
    soft_min: float | None = -3.402823e38,
    soft_max: float | None = 3.402823e38,
    step: float | None = 3,
    precision: int | None = 2,
    options: set[bpy.stub_internal.rna_enums.PropertyFlagItems] = {"ANIMATABLE"},
    override: set[bpy.stub_internal.rna_enums.PropertyOverrideFlagItems] = set(),
    tags: set[str] | None = set(),
    subtype: bpy.stub_internal.rna_enums.PropertySubtypeNumberItems = "NONE",
    unit: bpy.stub_internal.rna_enums.PropertyUnitItems = "NONE",
    update: collections.abc.Callable[[_GenericType1, bpy.types.Context], None]
    | None = None,
    get: collections.abc.Callable[[_GenericType1], float] | None = None,
    set: collections.abc.Callable[[_GenericType1, float], None] | None = None,
    get_transform: None
    | collections.abc.Callable[[bpy.types.bpy_struct, float, bool], float]
    | None = None,
    set_transform: None
    | collections.abc.Callable[[bpy.types.bpy_struct, float, float, bool], float]
    | None = None,
) -> None:
    """Returns a new float (single precision) property definition.

        :param name: Name used in the user interface.
        :param description: Text used for the tooltip and api documentation.
        :param translation_context: Text used as context to disambiguate translations.
        :param default: The default value for this property.
        :param min: Hard minimum, trying to assign a value below will silently assign this minimum instead.
        :param max: Hard maximum, trying to assign a value above will silently assign this maximum instead.
        :param soft_min: Soft minimum (>= min), user wont be able to drag the widget below this value in the UI.
        :param soft_max: Soft maximum (<= max), user wont be able to drag the widget above this value in the UI.
        :param step: Step of increment/decrement in UI, in [1, 100], defaults to 3 (WARNING: actual value is /100).
        :param precision: Maximum number of decimal digits to display, in [0, 6]. Fraction is automatically hidden for exact integer values of fields with unit NONE or TIME (frame count) and step divisible by 100.
        :param options: Enumerator in `rna_enum_property_flag_items`.
        :param override: Enumerator in `rna_enum_property_override_flag_items`.
        :param tags: Enumerator of tags that are defined by parent class.
        :param subtype: Enumerator in `rna_enum_property_subtype_number_items`.
        :param unit: Enumerator in `rna_enum_property_unit_items`.
        :param update: Function to be called when this value is modified,
    This function must take 2 values (self, context) and return None.
    Warning there are no safety checks to avoid infinite recursion.
        :param get: Function to be called when this value is read, and the default,
    system-defined storage is not used for this property.
    This function must take 1 value (self) and return the value of the property.

    Defining this callback without a matching set one will make the property read-only (even if READ_ONLY option is not set).
        :param set: Function to be called when this value is written, and the default,
    system-defined storage is not used for this property.
    This function must take 2 values (self, value) and return None.

    Defining this callback without a matching get one is invalid.
        :param get_transform: Function to be called when this value is read,
    if some additional processing must be performed on the stored value.
    This function must take three arguments (self, the stored value,
    and a boolean indicating if the property is currently set),
    and return the final, transformed value of the property.

    The callback is responsible to ensure that value limits of the property (min/max, length...) are respected. Otherwise a ValueError exception is raised.
        :param set_transform: Function to be called when this value is written,
    if some additional processing must be performed on the given value before storing it.
    This function must take four arguments (self, the given value to store,
    the currently stored value (raw value, without any get_transform applied to it),
    and a boolean indicating if the property is currently set),
    and return the final, transformed value of the property.

    The callback is responsible to ensure that value limits (min/max, length...) are respected. Otherwise a ValueError exception is raised.
    """

def FloatVectorProperty[_GenericType1: bpy.types.bpy_struct](
    *,
    name: str | None = "",
    description: str | None = "",
    translation_context: str | None = "*",
    default: collections.abc.Sequence[float] | None = (0.0, 0.0, 0.0),
    min: float | None = None,
    max: float | None = sys.float_info.max,
    soft_min: float | None = None,
    soft_max: float | None = sys.float_info.max,
    step: float | None = 3,
    precision: int | None = 2,
    options: set[bpy.stub_internal.rna_enums.PropertyFlagItems] = {"ANIMATABLE"},
    override: set[bpy.stub_internal.rna_enums.PropertyOverrideFlagItems] = set(),
    tags: set[str] | None = set(),
    subtype: bpy.stub_internal.rna_enums.PropertySubtypeNumberArrayItems = "NONE",
    unit: bpy.stub_internal.rna_enums.PropertyUnitItems = "NONE",
    size: collections.abc.Sequence[int] | int | None = 3,
    update: collections.abc.Callable[[_GenericType1, bpy.types.Context], None]
    | None = None,
    get: collections.abc.Callable[[_GenericType1], collections.abc.Sequence[float]]
    | None = None,
    set: collections.abc.Callable[
        [_GenericType1, collections.abc.Sequence[float]], None
    ]
    | None = None,
    get_transform: None
    | collections.abc.Callable[
        [bpy.types.bpy_struct, collections.abc.Sequence[float], bool],
        collections.abc.Sequence[float],
    ]
    | None = None,
    set_transform: None
    | collections.abc.Callable[
        [
            bpy.types.bpy_struct,
            collections.abc.Sequence[float],
            collections.abc.Sequence[float],
            bool,
        ],
        collections.abc.Sequence[float],
    ]
    | None = None,
) -> None:
    """Returns a new vector float property definition.

        :param name: Name used in the user interface.
        :param description: Text used for the tooltip and api documentation.
        :param translation_context: Text used as context to disambiguate translations.
        :param default: Sequence of floats the length of size.
        :param min: Hard minimum, trying to assign a value below will silently assign this minimum instead.
        :param max: Hard maximum, trying to assign a value above will silently assign this maximum instead.
        :param soft_min: Soft minimum (>= min), user wont be able to drag the widget below this value in the UI.
        :param soft_max: Soft maximum (<= max), user wont be able to drag the widget above this value in the UI.
        :param step: Step of increment/decrement in UI, in [1, 100], defaults to 3 (WARNING: actual value is /100).
        :param precision: Maximum number of decimal digits to display, in [0, 6]. Fraction is automatically hidden for exact integer values of fields with unit NONE or TIME (frame count) and step divisible by 100.
        :param options: Enumerator in `rna_enum_property_flag_items`.
        :param override: Enumerator in `rna_enum_property_override_flag_items`.
        :param tags: Enumerator of tags that are defined by parent class.
        :param subtype: Enumerator in `rna_enum_property_subtype_number_array_items`.
        :param unit: Enumerator in `rna_enum_property_unit_items`.
        :param size: Vector dimensions in [1, 32]. An int sequence can be used to define multi-dimension arrays.
        :param update: Function to be called when this value is modified,
    This function must take 2 values (self, context) and return None.
    Warning there are no safety checks to avoid infinite recursion.
        :param get: Function to be called when this value is read, and the default,
    system-defined storage is not used for this property.
    This function must take 1 value (self) and return the value of the property.

    Defining this callback without a matching set one will make the property read-only (even if READ_ONLY option is not set).
        :param set: Function to be called when this value is written, and the default,
    system-defined storage is not used for this property.
    This function must take 2 values (self, value) and return None.

    Defining this callback without a matching get one is invalid.
        :param get_transform: Function to be called when this value is read,
    if some additional processing must be performed on the stored value.
    This function must take three arguments (self, the stored value,
    and a boolean indicating if the property is currently set),
    and return the final, transformed value of the property.

    The callback is responsible to ensure that value limits of the property (min/max, length...) are respected. Otherwise a ValueError exception is raised.
        :param set_transform: Function to be called when this value is written,
    if some additional processing must be performed on the given value before storing it.
    This function must take four arguments (self, the given value to store,
    the currently stored value (raw value, without any get_transform applied to it),
    and a boolean indicating if the property is currently set),
    and return the final, transformed value of the property.

    The callback is responsible to ensure that value limits (min/max, length...) are respected. Otherwise a ValueError exception is raised.
    """

def IntProperty[_GenericType1: bpy.types.bpy_struct](
    *,
    name: str | None = "",
    description: str | None = "",
    translation_context: str | None = "*",
    default: int | None = 0,
    min: int | None = None,
    max: int | None = None,
    soft_min: int | None = None,
    soft_max: int | None = None,
    step: int | None = 1,
    options: set[bpy.stub_internal.rna_enums.PropertyFlagItems] = {"ANIMATABLE"},
    override: set[bpy.stub_internal.rna_enums.PropertyOverrideFlagItems] = set(),
    tags: set[str] | None = set(),
    subtype: bpy.stub_internal.rna_enums.PropertySubtypeNumberItems = "NONE",
    update: collections.abc.Callable[[_GenericType1, bpy.types.Context], None]
    | None = None,
    get: collections.abc.Callable[[_GenericType1], int] | None = None,
    set: collections.abc.Callable[[_GenericType1, int], None] | None = None,
    get_transform: None
    | collections.abc.Callable[[bpy.types.bpy_struct, int, bool], int]
    | None = None,
    set_transform: None
    | collections.abc.Callable[[bpy.types.bpy_struct, int, int, bool], int]
    | None = None,
) -> None:
    """Returns a new int property definition.

        :param name: Name used in the user interface.
        :param description: Text used for the tooltip and api documentation.
        :param translation_context: Text used as context to disambiguate translations.
        :param default: The default value for this property.
        :param min: Hard minimum, trying to assign a value below will silently assign this minimum instead.
        :param max: Hard maximum, trying to assign a value above will silently assign this maximum instead.
        :param soft_min: Soft minimum (>= min), user wont be able to drag the widget below this value in the UI.
        :param soft_max: Soft maximum (<= max), user wont be able to drag the widget above this value in the UI.
        :param step: Step of increment/decrement in UI, in [1, 100], defaults to 1 (WARNING: unused currently!).
        :param options: Enumerator in `rna_enum_property_flag_items`.
        :param override: Enumerator in `rna_enum_property_override_flag_items`.
        :param tags: Enumerator of tags that are defined by parent class.
        :param subtype: Enumerator in `rna_enum_property_subtype_number_items`.
        :param update: Function to be called when this value is modified,
    This function must take 2 values (self, context) and return None.
    Warning there are no safety checks to avoid infinite recursion.
        :param get: Function to be called when this value is read, and the default,
    system-defined storage is not used for this property.
    This function must take 1 value (self) and return the value of the property.

    Defining this callback without a matching set one will make the property read-only (even if READ_ONLY option is not set).
        :param set: Function to be called when this value is written, and the default,
    system-defined storage is not used for this property.
    This function must take 2 values (self, value) and return None.

    Defining this callback without a matching get one is invalid.
        :param get_transform: Function to be called when this value is read,
    if some additional processing must be performed on the stored value.
    This function must take three arguments (self, the stored value,
    and a boolean indicating if the property is currently set),
    and return the final, transformed value of the property.

    The callback is responsible to ensure that value limits of the property (min/max, length...) are respected. Otherwise a ValueError exception is raised.
        :param set_transform: Function to be called when this value is written,
    if some additional processing must be performed on the given value before storing it.
    This function must take four arguments (self, the given value to store,
    the currently stored value (raw value, without any get_transform applied to it),
    and a boolean indicating if the property is currently set),
    and return the final, transformed value of the property.

    The callback is responsible to ensure that value limits (min/max, length...) are respected. Otherwise a ValueError exception is raised.
    """

def IntVectorProperty[_GenericType1: bpy.types.bpy_struct](
    *,
    name: str | None = "",
    description: str | None = "",
    translation_context: str | None = "*",
    default: collections.abc.Sequence[int] | None = (0, 0, 0),
    min: int | None = None,
    max: int | None = None,
    soft_min: int | None = None,
    soft_max: int | None = None,
    step: int | None = 1,
    options: set[bpy.stub_internal.rna_enums.PropertyFlagItems] = {"ANIMATABLE"},
    override: set[bpy.stub_internal.rna_enums.PropertyOverrideFlagItems] = set(),
    tags: set[str] | None = set(),
    subtype: bpy.stub_internal.rna_enums.PropertySubtypeNumberArrayItems = "NONE",
    size: collections.abc.Sequence[int] | int | None = 3,
    update: collections.abc.Callable[[_GenericType1, bpy.types.Context], None]
    | None = None,
    get: collections.abc.Callable[[_GenericType1], collections.abc.Sequence[int]]
    | None = None,
    set: collections.abc.Callable[[_GenericType1, collections.abc.Sequence[int]], None]
    | None = None,
    get_transform: None
    | collections.abc.Callable[
        [bpy.types.bpy_struct, collections.abc.Sequence[int], bool],
        collections.abc.Sequence[int],
    ]
    | None = None,
    set_transform: None
    | collections.abc.Callable[
        [
            bpy.types.bpy_struct,
            collections.abc.Sequence[int],
            collections.abc.Sequence[int],
            bool,
        ],
        collections.abc.Sequence[int],
    ]
    | None = None,
) -> None:
    """Returns a new vector int property definition.

        :param name: Name used in the user interface.
        :param description: Text used for the tooltip and api documentation.
        :param translation_context: Text used as context to disambiguate translations.
        :param default: sequence of ints the length of size.
        :param min: Hard minimum, trying to assign a value below will silently assign this minimum instead.
        :param max: Hard maximum, trying to assign a value above will silently assign this maximum instead.
        :param soft_min: Soft minimum (>= min), user wont be able to drag the widget below this value in the UI.
        :param soft_max: Soft maximum (<= max), user wont be able to drag the widget above this value in the UI.
        :param step: Step of increment/decrement in UI, in [1, 100], defaults to 1 (WARNING: unused currently!).
        :param options: Enumerator in `rna_enum_property_flag_items`.
        :param override: Enumerator in `rna_enum_property_override_flag_items`.
        :param tags: Enumerator of tags that are defined by parent class.
        :param subtype: Enumerator in `rna_enum_property_subtype_number_array_items`.
        :param size: Vector dimensions in [1, 32]. An int sequence can be used to define multi-dimension arrays.
        :param update: Function to be called when this value is modified,
    This function must take 2 values (self, context) and return None.
    Warning there are no safety checks to avoid infinite recursion.
        :param get: Function to be called when this value is read, and the default,
    system-defined storage is not used for this property.
    This function must take 1 value (self) and return the value of the property.

    Defining this callback without a matching set one will make the property read-only (even if READ_ONLY option is not set).
        :param set: Function to be called when this value is written, and the default,
    system-defined storage is not used for this property.
    This function must take 2 values (self, value) and return None.

    Defining this callback without a matching get one is invalid.
        :param get_transform: Function to be called when this value is read,
    if some additional processing must be performed on the stored value.
    This function must take three arguments (self, the stored value,
    and a boolean indicating if the property is currently set),
    and return the final, transformed value of the property.

    The callback is responsible to ensure that value limits of the property (min/max, length...) are respected. Otherwise a ValueError exception is raised.
        :param set_transform: Function to be called when this value is written,
    if some additional processing must be performed on the given value before storing it.
    This function must take four arguments (self, the given value to store,
    the currently stored value (raw value, without any get_transform applied to it),
    and a boolean indicating if the property is currently set),
    and return the final, transformed value of the property.

    The callback is responsible to ensure that value limits (min/max, length...) are respected. Otherwise a ValueError exception is raised.
    """

def PointerProperty[_GenericType1: bpy.types.bpy_struct, _GenericType2: bpy.types.ID](
    *,
    type: type[bpy.types.PropertyGroup | bpy.types.ID] | None,
    name: str | None = "",
    description: str | None = "",
    translation_context: str | None = "*",
    options: set[bpy.stub_internal.rna_enums.PropertyFlagItems] = {"ANIMATABLE"},
    override: set[bpy.stub_internal.rna_enums.PropertyOverrideFlagItems] = set(),
    tags: set[str] | None = set(),
    poll: collections.abc.Callable[[_GenericType1, _GenericType2], bool] | None = None,
    update: collections.abc.Callable[[_GenericType1, bpy.types.Context], None]
    | None = None,
) -> None:
    """Returns a new pointer property definition.

        :param type: A subclass of PropertyGroup or ID.
        :param name: Name used in the user interface.
        :param description: Text used for the tooltip and api documentation.
        :param translation_context: Text used as context to disambiguate translations.
        :param options: Enumerator in `rna_enum_property_flag_items`.
        :param override: Enumerator in `rna_enum_property_override_flag_items`.
        :param tags: Enumerator of tags that are defined by parent class.
        :param poll: Function that determines whether an item is valid for this property.
    The function must take 2 values (self, object) and return a boolean.

    The return value will be checked only when assigning an item from the UI, but it is still possible to assign an "invalid" item to the property directly.
        :param update: Function to be called when this value is modified,
    This function must take 2 values (self, context) and return None.
    Warning there are no safety checks to avoid infinite recursion.
    """

def RemoveProperty(*, cls: type[bpy.types.bpy_struct] | None, attr: str | None) -> None:
    """Removes a dynamically defined property.

    :param cls: The class containing the property (must be a positional argument).
    :param attr: Property name (must be passed as a keyword).
    """

def StringProperty[_GenericType1: bpy.types.bpy_struct](
    *,
    name: str | None = "",
    description: str | None = "",
    translation_context: str | None = "*",
    default: str | None = "",
    maxlen: int | None = 0,
    options: set[bpy.stub_internal.rna_enums.PropertyFlagItems] = {"ANIMATABLE"},
    override: set[bpy.stub_internal.rna_enums.PropertyOverrideFlagItems] = set(),
    tags: set[str] | None = set(),
    subtype: bpy.stub_internal.rna_enums.PropertySubtypeStringItems = "NONE",
    update: collections.abc.Callable[[_GenericType1, bpy.types.Context], None]
    | None = None,
    get: collections.abc.Callable[[_GenericType1], str] | None = None,
    set: collections.abc.Callable[[_GenericType1, str], None] | None = None,
    get_transform: None
    | collections.abc.Callable[[bpy.types.bpy_struct, str, bool], str]
    | None = None,
    set_transform: None
    | collections.abc.Callable[[bpy.types.bpy_struct, str, str, bool], str]
    | None = None,
    search: collections.abc.Callable[
        [_GenericType1, bpy.types.Context, str],
        collections.abc.Iterable[str | tuple[str, str]],
    ]
    | None = None,
    search_options: set[str] | None = {"SUGGESTION"},
) -> None:
    """Returns a new string property definition.

        :param name: Name used in the user interface.
        :param description: Text used for the tooltip and api documentation.
        :param translation_context: Text used as context to disambiguate translations.
        :param default: initializer string.
        :param maxlen: maximum length of the string.
        :param options: Enumerator in `rna_enum_property_flag_items`.
        :param override: Enumerator in `rna_enum_property_override_flag_items`.
        :param tags: Enumerator of tags that are defined by parent class.
        :param subtype: Enumerator in `rna_enum_property_subtype_string_items`.
        :param update: Function to be called when this value is modified,
    This function must take 2 values (self, context) and return None.
    Warning there are no safety checks to avoid infinite recursion.
        :param get: Function to be called when this value is read, and the default,
    system-defined storage is not used for this property.
    This function must take 1 value (self) and return the value of the property.

    Defining this callback without a matching set one will make the property read-only (even if READ_ONLY option is not set).
        :param set: Function to be called when this value is written, and the default,
    system-defined storage is not used for this property.
    This function must take 2 values (self, value) and return None.

    Defining this callback without a matching get one is invalid.
        :param get_transform: Function to be called when this value is read,
    if some additional processing must be performed on the stored value.
    This function must take three arguments (self, the stored value,
    and a boolean indicating if the property is currently set),
    and return the final, transformed value of the property.

    The callback is responsible to ensure that value limits of the property (min/max, length...) are respected. Otherwise a ValueError exception is raised.
        :param set_transform: Function to be called when this value is written,
    if some additional processing must be performed on the given value before storing it.
    This function must take four arguments (self, the given value to store,
    the currently stored value (raw value, without any get_transform applied to it),
    and a boolean indicating if the property is currently set),
    and return the final, transformed value of the property.

    The callback is responsible to ensure that value limits (min/max, length...) are respected. Otherwise a ValueError exception is raised.
        :param search: Function to be called to show candidates for this string (shown in the UI).
    This function must take 3 values (self, context, edit_text)
    and return a sequence, iterator or generator where each item must be:

    A single string (representing a candidate to display).

    A tuple-pair of strings, where the first is a candidate and the second
    is additional information about the candidate.
        :param search_options: Set of strings in:

    SORT sorts the resulting items.

    SUGGESTION lets the user enter values not found in search candidates.
    WARNING disabling this flag causes the search callback to run on redraw,
    so only disable this flag if its not likely to cause performance issues.
    """



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import _bpy_types
import bpy.types

class PHYSICS_PT_rigid_body(PHYSICS_PT_rigidbody_panel, _bpy_types.Panel):
    COMPAT_ENGINES: typing.Any
    bl_context: typing.Any
    bl_label: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

    @classmethod
    def poll(cls, context) -> None:
        """

        :param context:
        """

class PHYSICS_PT_rigid_body_collisions(PHYSICS_PT_rigidbody_panel, _bpy_types.Panel):
    COMPAT_ENGINES: typing.Any
    bl_context: typing.Any
    bl_label: typing.Any
    bl_parent_id: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

    @classmethod
    def poll(cls, context) -> None:
        """

        :param context:
        """

class PHYSICS_PT_rigid_body_collisions_collections(
    PHYSICS_PT_rigidbody_panel, _bpy_types.Panel
):
    COMPAT_ENGINES: typing.Any
    bl_context: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_parent_id: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

    @classmethod
    def poll(cls, context) -> None:
        """

        :param context:
        """

class PHYSICS_PT_rigid_body_collisions_sensitivity(
    PHYSICS_PT_rigidbody_panel, _bpy_types.Panel
):
    COMPAT_ENGINES: typing.Any
    bl_context: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_parent_id: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

    @classmethod
    def poll(cls, context) -> None:
        """

        :param context:
        """

class PHYSICS_PT_rigid_body_collisions_surface(
    PHYSICS_PT_rigidbody_panel, _bpy_types.Panel
):
    COMPAT_ENGINES: typing.Any
    bl_context: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_parent_id: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

    @classmethod
    def poll(cls, context) -> None:
        """

        :param context:
        """

class PHYSICS_PT_rigid_body_dynamics(PHYSICS_PT_rigidbody_panel, _bpy_types.Panel):
    COMPAT_ENGINES: typing.Any
    bl_context: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_parent_id: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

    @classmethod
    def poll(cls, context) -> None:
        """

        :param context:
        """

class PHYSICS_PT_rigid_body_dynamics_deactivation(
    PHYSICS_PT_rigidbody_panel, _bpy_types.Panel
):
    COMPAT_ENGINES: typing.Any
    bl_context: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_parent_id: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

    def draw_header(self, context) -> None:
        """

        :param context:
        """

    @classmethod
    def poll(cls, context) -> None:
        """

        :param context:
        """

class PHYSICS_PT_rigid_body_settings(PHYSICS_PT_rigidbody_panel, _bpy_types.Panel):
    COMPAT_ENGINES: typing.Any
    bl_context: typing.Any
    bl_label: typing.Any
    bl_parent_id: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

    @classmethod
    def poll(cls, context) -> None:
        """

        :param context:
        """

class PHYSICS_PT_rigidbody_panel:
    bl_context: typing.Any
    bl_region_type: typing.Any
    bl_space_type: typing.Any

def rigid_body_warning(layout, text) -> None: ...



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt

class IDPropertyArray:
    typecode: typing.Any
    """ The type of the data in the array {'f': float, 'd': double, 'i': int, 'b': bool}."""

    def to_list(self) -> list[float] | list[int]:
        """Return the array as a list.

        :return: The array as a list.
        """

class IDPropertyGroup:
    name: typing.Any
    """ The name of this Group."""

    def clear(self) -> None:
        """Clear all members from this group."""

    def get(self, key: str, default: typing.Any | None = None) -> typing.Any:
        """Return the value for key, if it exists, else default.

        :param key: The key to look up.
        :param default: Value to return if key is not found.
        :return: The value for the key, or default if not found.
        """

    def items(self) -> IDPropertyGroupViewItems:
        """Iterate through the items in the dict; behaves like dictionary method items.

        :return: A view of the items.
        """

    def keys(self) -> IDPropertyGroupViewKeys:
        """Return the keys associated with this group.

        :return: The keys.
        """

    def pop(self, key: str, default: typing.Any | None = None) -> typing.Any:
        """Remove an item from the group, returning a Python representation.

        :param key: Name of item to remove.
        :param default: Value to return when key isnt found, otherwise raise an exception.
        :return: A Python representation of the removed item.
        """

    def to_dict(self) -> dict[str, typing.Any]:
        """Return a purely Python version of the group.

        :return: A dictionary representation of the group.
        """

    def update(self, other: dict[str, typing.Any] | typing_extensions.Self) -> None:
        """Update key-value pairs.

        :param other: Updates the values in the group with this.
        """

    def values(self) -> IDPropertyGroupViewValues:
        """Return the values associated with this group.

        :return: A view of the values.
        """

class IDPropertyGroupIterItems: ...
class IDPropertyGroupIterKeys: ...
class IDPropertyGroupIterValues: ...
class IDPropertyGroupViewItems(collections.abc.Iterable[tuple[str, typing.Any]]): ...
class IDPropertyGroupViewKeys(collections.abc.Iterable[str]): ...
class IDPropertyGroupViewValues(collections.abc.Iterable[typing.Any]): ...



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import _bpy_types
import bpy.types

class AddModifierMenu(_bpy_types.Operator):
    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def invoke(self, _context, _event) -> None:
        """

        :param _context:
        :param _event:
        """

    @classmethod
    def poll(cls, context) -> None:
        """

        :param context:
        """

class DATA_PT_modifiers(ModifierButtonsPanel, _bpy_types.Panel):
    bl_context: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, _context) -> None:
        """

        :param _context:
        """

    @classmethod
    def poll(cls, context) -> None:
        """

        :param context:
        """

class ModifierAddMenu:
    MODIFIER_TYPES_I18N_CONTEXT: typing.Any
    MODIFIER_TYPES_TO_ICONS: typing.Any
    MODIFIER_TYPES_TO_LABELS: typing.Any

    @classmethod
    def operator_modifier_add(cls, layout, mod_type, text=None, no_icon=False) -> None:
        """

        :param layout:
        :param mod_type:
        :param text:
        :param no_icon:
        """

    @classmethod
    def operator_modifier_add_asset(cls, layout, name, icon="NONE") -> None:
        """

        :param layout:
        :param name:
        :param icon:
        """

class ModifierButtonsPanel:
    bl_context: typing.Any
    bl_options: typing.Any
    bl_region_type: typing.Any
    bl_space_type: typing.Any

class OBJECT_MT_modifier_add(ModifierAddMenu, _bpy_types.Menu):
    MODIFIER_TYPES_I18N_CONTEXT: typing.Any
    MODIFIER_TYPES_TO_ICONS: typing.Any
    MODIFIER_TYPES_TO_LABELS: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

class OBJECT_MT_modifier_add_color(ModifierAddMenu, _bpy_types.Menu):
    MODIFIER_TYPES_I18N_CONTEXT: typing.Any
    MODIFIER_TYPES_TO_ICONS: typing.Any
    MODIFIER_TYPES_TO_LABELS: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

class OBJECT_MT_modifier_add_deform(ModifierAddMenu, _bpy_types.Menu):
    MODIFIER_TYPES_I18N_CONTEXT: typing.Any
    MODIFIER_TYPES_TO_ICONS: typing.Any
    MODIFIER_TYPES_TO_LABELS: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

class OBJECT_MT_modifier_add_edit(ModifierAddMenu, _bpy_types.Menu):
    MODIFIER_TYPES_I18N_CONTEXT: typing.Any
    MODIFIER_TYPES_TO_ICONS: typing.Any
    MODIFIER_TYPES_TO_LABELS: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

class OBJECT_MT_modifier_add_generate(ModifierAddMenu, _bpy_types.Menu):
    MODIFIER_TYPES_I18N_CONTEXT: typing.Any
    MODIFIER_TYPES_TO_ICONS: typing.Any
    MODIFIER_TYPES_TO_LABELS: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

class OBJECT_MT_modifier_add_normals(ModifierAddMenu, _bpy_types.Menu):
    MODIFIER_TYPES_I18N_CONTEXT: typing.Any
    MODIFIER_TYPES_TO_ICONS: typing.Any
    MODIFIER_TYPES_TO_LABELS: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """

class OBJECT_MT_modifier_add_physics(ModifierAddMenu, _bpy_types.Menu):
    MODIFIER_TYPES_I18N_CONTEXT: typing.Any
    MODIFIER_TYPES_TO_ICONS: typing.Any
    MODIFIER_TYPES_TO_LABELS: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def draw(self, context) -> None:
        """

        :param context:
        """
